@@ -15,6 +15,9 @@
 #include "core/fpdfapi/parser/cpdf_document.h"
 #include "core/fpdfapi/parser/cpdf_object.h"
 #include "core/fpdfapi/parser/cpdf_stream.h"
+#include "core/fxcrt/fx_safe_types.h"
+#include "third_party/base/check.h"
+#include "third_party/base/notreached.h"
 
 namespace {
 
@@ -31,7 +34,7 @@ CPDF_ShadingPattern::CPDF_ShadingPattern(CPDF_Document* pDoc,
                                          bool bShading,
                                          const CFX_Matrix& parentMatrix)
     : CPDF_Pattern(pDoc, pPatternObj, parentMatrix), m_bShading(bShading) {
-  ASSERT(document());
+  DCHECK(document());
   if (!bShading)
     SetPatternToFormMatrix();
 }
@@ -151,7 +154,7 @@ bool CPDF_ShadingPattern::ValidateFunctions(
   if (m_pFunctions.size() != nExpectedNumFunctions)
     return false;
 
-  pdfium::base::CheckedNumeric<uint32_t> nTotalOutputs = 0;
+  FX_SAFE_UINT32 nTotalOutputs = 0;
   for (const auto& function : m_pFunctions) {
     if (!function)
       return false;
