@@ -13,7 +13,7 @@
 #include "core/fxcrt/observed_ptr.h"
 #include "core/fxcrt/retain_ptr.h"
 #include "core/fxcrt/unowned_ptr.h"
-#include "core/fxge/fx_dib.h"
+#include "core/fxge/dib/fx_dib.h"
 #include "third_party/base/span.h"
 
 class CPDF_Document;
@@ -21,8 +21,7 @@ class CFX_DIBBase;
 
 class CPDF_TransferFunc final : public Retainable, public Observable {
  public:
-  template <typename T, typename... Args>
-  friend RetainPtr<T> pdfium::MakeRetain(Args&&... args);
+  CONSTRUCT_VIA_MAKE_RETAIN;
 
   static constexpr size_t kChannelSampleSize = 256;
 
@@ -39,14 +38,14 @@ class CPDF_TransferFunc final : public Retainable, public Observable {
   bool GetIdentity() const { return m_bIdentity; }
 
  private:
-  CPDF_TransferFunc(CPDF_Document* pDoc,
+  CPDF_TransferFunc(const CPDF_Document* pDoc,
                     bool bIdentify,
                     std::vector<uint8_t, FxAllocAllocator<uint8_t>> samples_r,
                     std::vector<uint8_t, FxAllocAllocator<uint8_t>> samples_g,
                     std::vector<uint8_t, FxAllocAllocator<uint8_t>> samples_b);
   ~CPDF_TransferFunc() override;
 
-  UnownedPtr<CPDF_Document> const m_pPDFDoc;
+  UnownedPtr<const CPDF_Document> const m_pPDFDoc;
   const bool m_bIdentity;
   const std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_SamplesR;
   const std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_SamplesG;
