@@ -10,9 +10,11 @@
 #include <vector>
 
 #include "core/fxcrt/fx_coordinates.h"
+#include "core/fxcrt/fx_memory_wrappers.h"
 #include "core/fxcrt/retain_ptr.h"
 #include "core/fxcrt/unowned_ptr.h"
-#include "core/fxge/fx_dib.h"
+#include "core/fxge/dib/fx_dib.h"
+#include "third_party/base/span.h"
 
 class CFX_DIBBase;
 class PauseIndicatorIface;
@@ -58,10 +60,10 @@ class CStretchEngine {
     size_t GetPixelWeightSize() const;
 
    private:
-    int m_DestMin;
-    int m_ItemSize;
-    std::vector<uint8_t> m_WeightTables;
-    size_t m_dwWeightTablesSize;
+    int m_DestMin = 0;
+    int m_ItemSize = 0;
+    size_t m_dwWeightTablesSize = 0;
+    std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_WeightTables;
   };
 
   enum class State : uint8_t { kInitial, kHorizontal, kVertical };
@@ -82,17 +84,17 @@ class CStretchEngine {
   const int m_SrcBpp;
   const int m_bHasAlpha;
   RetainPtr<CFX_DIBBase> const m_pSource;
-  const uint32_t* m_pSrcPalette;
+  pdfium::span<const uint32_t> m_pSrcPalette;
   const int m_SrcWidth;
   const int m_SrcHeight;
   UnownedPtr<ScanlineComposerIface> const m_pDestBitmap;
   const int m_DestWidth;
   const int m_DestHeight;
   const FX_RECT m_DestClip;
-  std::vector<uint8_t> m_DestScanline;
-  std::vector<uint8_t> m_DestMaskScanline;
-  std::vector<uint8_t> m_InterBuf;
-  std::vector<uint8_t> m_ExtraAlphaBuf;
+  std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_DestScanline;
+  std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_DestMaskScanline;
+  std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_InterBuf;
+  std::vector<uint8_t, FxAllocAllocator<uint8_t>> m_ExtraAlphaBuf;
   FX_RECT m_SrcClip;
   int m_InterPitch;
   int m_ExtraMaskPitch;
