@@ -7,6 +7,8 @@
 
 #include "tests/Test.h"
 
+#include "include/gpu/GrDirectContext.h"
+
 using sk_gpu_test::GrContextFactory;
 using sk_gpu_test::GLTestContext;
 using sk_gpu_test::ContextInfo;
@@ -22,13 +24,15 @@ bool IsVulkanContextType(sk_gpu_test::GrContextFactory::ContextType type) {
 bool IsMetalContextType(sk_gpu_test::GrContextFactory::ContextType type) {
     return GrBackendApi::kMetal == GrContextFactory::ContextTypeBackend(type);
 }
+bool IsDirect3DContextType(sk_gpu_test::GrContextFactory::ContextType type) {
+    return GrBackendApi::kDirect3D == GrContextFactory::ContextTypeBackend(type);
+}
+bool IsDawnContextType(sk_gpu_test::GrContextFactory::ContextType type) {
+    return GrBackendApi::kDawn == GrContextFactory::ContextTypeBackend(type);
+}
 bool IsRenderingGLContextType(sk_gpu_test::GrContextFactory::ContextType type) {
     return IsGLContextType(type) && GrContextFactory::IsRenderingContext(type);
 }
-bool IsRenderingGLOrMetalContextType(sk_gpu_test::GrContextFactory::ContextType type) {
-    return (IsGLContextType(type) || IsMetalContextType(type)) &&
-           GrContextFactory::IsRenderingContext(type);
-}
 bool IsMockContextType(sk_gpu_test::GrContextFactory::ContextType type) {
     return type == GrContextFactory::kMock_ContextType;
 }
@@ -62,12 +66,13 @@ void RunWithGPUTestContexts(GrContextTestFn* test, GrContextTypeFilterFn* contex
         }
 
         ReporterContext ctx(reporter, SkString(GrContextFactory::ContextTypeName(contextType)));
-        if (ctxInfo.grContext()) {
+        if (ctxInfo.directContext()) {
             (*test)(reporter, ctxInfo);
             // In case the test changed the current context make sure we move it back before
             // calling flush.
             ctxInfo.testContext()->makeCurrent();
-            ctxInfo.grContext()->flush();
+            // Sync so any release/finished procs get called.
+            ctxInfo.directContext()->flushAndSubmit(/*sync*/true);
         }
     }
 }
