@@ -13,6 +13,7 @@
 #include "core/fxcrt/cfx_timer.h"
 #include "core/fxcrt/observed_ptr.h"
 #include "core/fxcrt/unowned_ptr.h"
+#include "core/fxcrt/widestring.h"
 #include "core/fxge/cfx_color.h"
 #include "core/fxge/cfx_renderdevice.h"
 #include "fpdfsdk/pwl/ipwl_systemhandler.h"
@@ -24,10 +25,8 @@ class IPVT_FontMap;
 struct PWL_SCROLL_INFO;
 
 // window styles
-#define PWS_CHILD 0x80000000L
 #define PWS_BORDER 0x40000000L
 #define PWS_BACKGROUND 0x20000000L
-#define PWS_HSCROLL 0x10000000L
 #define PWS_VSCROLL 0x08000000L
 #define PWS_VISIBLE 0x04000000L
 #define PWS_READONLY 0x01000000L
@@ -42,16 +41,13 @@ struct PWL_SCROLL_INFO;
 #define PES_RIGHT 0x0008L
 #define PES_MIDDLE 0x0010L
 #define PES_TOP 0x0020L
-#define PES_BOTTOM 0x0040L
 #define PES_CENTER 0x0080L
 #define PES_CHARARRAY 0x0100L
 #define PES_AUTOSCROLL 0x0200L
 #define PES_AUTORETURN 0x0400L
 #define PES_UNDO 0x0800L
 #define PES_RICH 0x1000L
-#define PES_SPELLCHECK 0x2000L
 #define PES_TEXTOVERFLOW 0x4000L
-#define PES_NOREAD 0x8000L
 
 // listbox styles
 #define PLBS_MULTIPLESEL 0x0001L
@@ -112,48 +108,54 @@ class CPWL_Wnd : public Observable {
     CreateParams(const CreateParams& other);
     ~CreateParams();
 
-    CFX_FloatRect rcRectWnd;                          // required
-    UnownedPtr<TimerHandlerIface> pTimerHandler;      // required
-    UnownedPtr<IPWL_SystemHandler> pSystemHandler;    // required
-    UnownedPtr<IPVT_FontMap> pFontMap;                // required
-    ObservedPtr<ProviderIface> pProvider;             // required
-    UnownedPtr<FocusHandlerIface> pFocusHandler;      // optional
-    uint32_t dwFlags = 0;                             // optional
-    CFX_Color sBackgroundColor;                       // optional
-    BorderStyle nBorderStyle = BorderStyle::SOLID;    // optional
-    int32_t dwBorderWidth = 1;                        // optional
-    CFX_Color sBorderColor;                           // optional
-    CFX_Color sTextColor;                             // optional
-    int32_t nTransparency = 255;                      // optional
-    float fFontSize;                                  // optional
-    CPWL_Dash sDash;                                  // optional
-    CPWL_MsgControl* pMsgControl = nullptr;           // ignore
-    int32_t eCursorType = FXCT_ARROW;                 // ignore
-    CFX_Matrix mtChild;                               // ignore
+    // Required:
+    CFX_FloatRect rcRectWnd;
+    ObservedPtr<CFX_Timer::HandlerIface> pTimerHandler;
+    UnownedPtr<IPWL_SystemHandler> pSystemHandler;
+    UnownedPtr<IPVT_FontMap> pFontMap;
+    ObservedPtr<ProviderIface> pProvider;
+    // Optional:
+    UnownedPtr<FocusHandlerIface> pFocusHandler;
+    uint32_t dwFlags = 0;
+    CFX_Color sBackgroundColor;
+    BorderStyle nBorderStyle = BorderStyle::kSolid;
+    int32_t dwBorderWidth = 1;
+    CFX_Color sBorderColor;
+    CFX_Color sTextColor;
+    int32_t nTransparency = 255;
+    float fFontSize;
+    CPWL_Dash sDash;
+    // Ignore:
+    CPWL_MsgControl* pMsgControl = nullptr;
+    int32_t eCursorType = FXCT_ARROW;
   };
 
   static bool IsSHIFTKeyDown(uint32_t nFlag);
   static bool IsCTRLKeyDown(uint32_t nFlag);
   static bool IsALTKeyDown(uint32_t nFlag);
+  static bool IsMETAKeyDown(uint32_t nFlag);
+
+  // Selects between IsCTRLKeyDown() and IsMETAKeyDown() depending on platform.
+  static bool IsPlatformShortcutKey(uint32_t nFlag);
 
   CPWL_Wnd(const CreateParams& cp,
            std::unique_ptr<IPWL_SystemHandler::PerWindowData> pAttachedData);
   virtual ~CPWL_Wnd();
 
   // Returns |true| iff this instance is still allocated.
-  virtual bool InvalidateRect(CFX_FloatRect* pRect);
+  virtual bool InvalidateRect(const CFX_FloatRect* pRect);
 
   virtual bool OnKeyDown(uint16_t nChar, uint32_t nFlag);
   virtual bool OnChar(uint16_t nChar, uint32_t nFlag);
-  virtual bool OnLButtonDblClk(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnLButtonDown(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnLButtonUp(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnRButtonDown(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnRButtonUp(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnMouseMove(const CFX_PointF& point, uint32_t nFlag);
-  virtual bool OnMouseWheel(short zDelta,
+  virtual bool OnLButtonDblClk(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnLButtonDown(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnLButtonUp(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnRButtonDown(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnRButtonUp(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnMouseMove(uint32_t nFlag, const CFX_PointF& point);
+  virtual bool OnMouseWheel(uint32_t nFlag,
                             const CFX_PointF& point,
-                            uint32_t nFlag);
+                            const CFX_Vector& delta);
   virtual void SetScrollInfo(const PWL_SCROLL_INFO& info);
   virtual void SetScrollPosition(float pos);
   virtual void ScrollWindowVertically(float pos);
@@ -172,6 +174,7 @@ class CPWL_Wnd : public Observable {
   virtual WideString GetText();
   virtual WideString GetSelectedText();
   virtual void ReplaceSelection(const WideString& text);
+  virtual bool SelectAllText();
 
   virtual bool CanUndo();
   virtual bool CanRedo();
@@ -201,8 +204,6 @@ class CPWL_Wnd : public Observable {
   void SetTextColor(const CFX_Color& color);
   CFX_Color GetBorderLeftTopColor(BorderStyle nBorderStyle) const;
   CFX_Color GetBorderRightBottomColor(BorderStyle nBorderStyle) const;
-
-  void SetBorderStyle(BorderStyle nBorderStyle);
   BorderStyle GetBorderStyle() const;
   const CPWL_Dash& GetBorderDash() const;
 
@@ -229,8 +230,6 @@ class CPWL_Wnd : public Observable {
   bool ClientHitTest(const CFX_PointF& point) const;
   bool IsCaptureMouse() const;
 
-  void EnableWindow(bool bEnable);
-  bool IsEnabled() const { return m_bEnabled; }
   const CPWL_Wnd* GetFocused() const;
   bool IsFocused() const;
   bool IsReadOnly() const;
@@ -247,9 +246,6 @@ class CPWL_Wnd : public Observable {
   int32_t GetTransparency();
   void SetTransparency(int32_t nTransparency);
 
-  CFX_Matrix GetChildToRoot() const;
-  CFX_Matrix GetChildMatrix() const;
-  void SetChildMatrix(const CFX_Matrix& mt);
   CFX_Matrix GetWindowMatrix() const;
 
   virtual void OnSetFocus();
@@ -270,7 +266,7 @@ class CPWL_Wnd : public Observable {
   bool IsNotifying() const { return m_bNotifying; }
   bool IsValid() const { return m_bCreated; }
   CreateParams* GetCreationParams() { return &m_CreationParams; }
-  TimerHandlerIface* GetTimerHandler() const {
+  CFX_Timer::HandlerIface* GetTimerHandler() const {
     return m_CreationParams.pTimerHandler.Get();
   }
   IPWL_SystemHandler* GetSystemHandler() const {
@@ -295,9 +291,6 @@ class CPWL_Wnd : public Observable {
   }
 
  private:
-  CFX_PointF ParentToChild(const CFX_PointF& point) const;
-  CFX_FloatRect ParentToChild(const CFX_FloatRect& rect) const;
-
   void DrawChildAppearance(CFX_RenderDevice* pDevice,
                            const CFX_Matrix& mtUser2Device);
 
@@ -322,7 +315,6 @@ class CPWL_Wnd : public Observable {
   bool m_bCreated = false;
   bool m_bVisible = false;
   bool m_bNotifying = false;
-  bool m_bEnabled = true;
 };
 
 #endif  // FPDFSDK_PWL_CPWL_WND_H_
