@@ -9,32 +9,25 @@
 
 #include "core/fxcrt/fx_coordinates.h"
 #include "core/fxcrt/fx_system.h"
-#include "core/fxge/fx_dib.h"
 #include "fxbarcode/cbc_onecode.h"
 
-class CBC_Codabar : public CBC_OneCode {
+class CBC_OnedCodaBarWriter;
+
+class CBC_Codabar final : public CBC_OneCode {
  public:
   CBC_Codabar();
   ~CBC_Codabar() override;
 
   // CBC_OneCode:
-  bool Encode(const CFX_WideStringC& contents,
-              bool isDevice,
-              int32_t& e) override;
-  bool RenderDevice(CFX_RenderDevice* device,
-                    const CFX_Matrix* matrix,
-                    int32_t& e) override;
-  bool RenderBitmap(CFX_RetainPtr<CFX_DIBitmap>& pOutBitmap,
-                    int32_t& e) override;
   BC_TYPE GetType() override;
-
-  bool SetStartChar(char start);
-  bool SetEndChar(char end);
-  bool SetTextLocation(BC_TEXT_LOC location);
-  bool SetWideNarrowRatio(int32_t ratio);
+  bool Encode(WideStringView contents) override;
+  bool RenderDevice(CFX_RenderDevice* device,
+                    const CFX_Matrix& matrix) override;
 
  private:
-  CFX_WideString m_renderContents;
+  CBC_OnedCodaBarWriter* GetOnedCodaBarWriter();
+
+  WideString m_renderContents;
 };
 
 #endif  // FXBARCODE_CBC_CODABAR_H_
