@@ -23,10 +23,12 @@
 #include "include/core/SkSurface.h"
 #include "include/core/SkTypeface.h"
 #include "include/core/SkTypes.h"
-#include "include/gpu/GrContext.h"
+#include "include/gpu/GrDirectContext.h"
+#include "include/gpu/GrRecordingContext.h"
 #include "include/gpu/GrTypes.h"
 #include "include/private/SkTArray.h"
-#include "src/gpu/GrContextPriv.h"
+#include "src/gpu/GrDirectContextPriv.h"
+#include "src/gpu/GrPixmap.h"
 #include "src/image/SkImage_Base.h"
 #include "src/image/SkImage_Gpu.h"
 #include "tools/ToolUtils.h"
@@ -35,7 +37,7 @@
 #include <string.h>
 #include <utility>
 
-class GrRenderTargetContext;
+class GrSurfaceDrawContext;
 
 static const int kNumMatrices = 6;
 static const int kImageSize = 128;
@@ -80,7 +82,7 @@ static const SkMatrix kUVMatrices[kNumMatrices] = {
 
 
 // Create a fixed size text label like "LL" or "LR".
-static sk_sp<SkImage> make_text_image(GrContext* context, const char* text, SkColor color) {
+static sk_sp<SkImage> make_text_image(GrDirectContext* direct, const char* text, SkColor color) {
     SkPaint paint;
     paint.setAntiAlias(true);
     paint.setColor(color);
@@ -92,8 +94,7 @@ static sk_sp<SkImage> make_text_image(GrContext* context, const char* text, SkCo
 
     SkRect bounds;
     font.measureText(text, strlen(text), SkTextEncoding::kUTF8, &bounds);
-    const SkMatrix mat = SkMatrix::MakeRectToRect(bounds, SkRect::MakeWH(kLabelSize, kLabelSize),
-                                                  SkMatrix::kFill_ScaleToFit);
+    const SkMatrix mat = SkMatrix::RectToRect(bounds, SkRect::MakeWH(kLabelSize, kLabelSize));
 
     const SkImageInfo ii = SkImageInfo::MakeN32Premul(kLabelSize, kLabelSize);
     sk_sp<SkSurface> surf = SkSurface::MakeRaster(ii);
@@ -106,12 +107,12 @@ static sk_sp<SkImage> make_text_image(GrContext* context, const char* text, SkCo
 
     sk_sp<SkImage> image = surf->makeImageSnapshot();
 
-    return image->makeTextureImage(context);
+    return image->makeTextureImage(direct);
 }
 
 // Create an image with each corner marked w/ "LL", "LR", etc., with the origin either bottom-left
 // or top-left.
-static sk_sp<SkImage> make_reference_image(GrContext* context,
+static sk_sp<SkImage> make_reference_image(GrDirectContext* context,
                                            const SkTArray<sk_sp<SkImage>>& labels,
                                            bool bottomLeftOrigin) {
     SkASSERT(kNumLabels == labels.count());
@@ -131,16 +132,12 @@ static sk_sp<SkImage> make_reference_image(GrContext* context,
 
     auto origin = bottomLeftOrigin ? kBottomLeft_GrSurfaceOrigin : kTopLeft_GrSurfaceOrigin;
 
-    // TODO: make MakeTextureProxyFromData return a GrSurfaceProxyView
-    auto proxy = sk_gpu_test::MakeTextureProxyFromData(context, GrRenderable::kNo, origin,
-                                                       bm.info(), bm.getPixels(), bm.rowBytes());
-    if (!proxy) {
+    auto view = sk_gpu_test::MakeTextureProxyViewFromData(context, GrRenderable::kNo, origin,
+                                                          bm.pixmap());
+    if (!view) {
         return nullptr;
     }
 
-    GrSwizzle swizzle = context->priv().caps()->getReadSwizzle(proxy->backendFormat(),
-                                                               GrColorType::kRGBA_8888);
-    GrSurfaceProxyView view(std::move(proxy), origin, swizzle);
     return sk_make_sp<SkImage_Gpu>(sk_ref_sp(context), kNeedNewImageUniqueID, std::move(view),
                                    ii.colorType(), kOpaque_SkAlphaType, nullptr);
 }
@@ -203,7 +200,8 @@ private:
                 canvas->drawImageRect(image, kSubsets[matIndex],
                                       drawScaled ? SkRect::MakeWH(kImageSize, kImageSize)
                                                  : kSubsets[matIndex],
-                                      nullptr, SkCanvas::kFast_SrcRectConstraint);
+                                      SkSamplingOptions(), nullptr,
+                                      SkCanvas::kFast_SrcRectConstraint);
             } else {
                 canvas->drawImage(image, 0, 0);
             }
@@ -217,23 +215,20 @@ private:
         canvas->restore();
     }
 
-    void drawRow(GrContext* context, SkCanvas* canvas,
-                 bool bottomLeftImage, bool drawSubset, bool drawScaled) {
-
-        sk_sp<SkImage> referenceImage = make_reference_image(context, fLabels, bottomLeftImage);
+    void drawRow(SkCanvas* canvas, bool bottomLeftImage, bool drawSubset, bool drawScaled) {
 
         canvas->save();
             canvas->translate(kLabelSize, kLabelSize);
 
             for (int i = 0; i < kNumMatrices; ++i) {
-                this->drawImageWithMatrixAndLabels(canvas, referenceImage.get(), i,
-                                                   drawSubset, drawScaled);
+                this->drawImageWithMatrixAndLabels(canvas, fReferenceImages[bottomLeftImage].get(),
+                                                   i, drawSubset, drawScaled);
                 canvas->translate(kCellSize, 0);
             }
         canvas->restore();
     }
 
-    void makeLabels(GrContext* context) {
+    void makeLabels(GrDirectContext* direct) {
         if (fLabels.count()) {
             return;
         }
@@ -248,33 +243,54 @@ private:
         };
 
         for (int i = 0; i < kNumLabels; ++i) {
-            fLabels.push_back(make_text_image(context, kLabelText[i], kLabelColors[i]));
+            fLabels.push_back(make_text_image(direct, kLabelText[i], kLabelColors[i]));
         }
         SkASSERT(kNumLabels == fLabels.count());
     }
 
-    void onDraw(GrContext* context, GrRenderTargetContext*, SkCanvas* canvas) override {
+    DrawResult onGpuSetup(GrDirectContext* context, SkString* errorMsg) override {
+        if (!context || context->abandoned()) {
+            return DrawResult::kSkip;
+        }
+
         this->makeLabels(context);
+        fReferenceImages[0] = make_reference_image(context, fLabels, false);
+        fReferenceImages[1] = make_reference_image(context, fLabels, true);
+        if (!fReferenceImages[0] || !fReferenceImages[1]) {
+            *errorMsg = "Failed to create reference images.";
+            return DrawResult::kFail;
+        }
+
+        return DrawResult::kOk;
+    }
+
+    void onGpuTeardown() override {
+        fLabels.reset();
+        fReferenceImages[0] = fReferenceImages[1] = nullptr;
+    }
+
+    void onDraw(GrRecordingContext*, GrSurfaceDrawContext*, SkCanvas* canvas) override {
+        SkASSERT(fReferenceImages[0] && fReferenceImages[1]);
 
         canvas->save();
 
         // Top row gets TL image
-        this->drawRow(context, canvas, false, false, false);
+        this->drawRow(canvas, false, false, false);
 
         canvas->translate(0, kCellSize);
 
         // Bottom row gets BL image
-        this->drawRow(context, canvas, true, false, false);
+        this->drawRow(canvas, true, false, false);
 
         canvas->translate(0, kCellSize);
 
         // Third row gets subsets of BL images
-        this->drawRow(context, canvas, true, true, false);
+        this->drawRow(canvas, true, true, false);
 
         canvas->translate(0, kCellSize);
 
         // Fourth row gets scaled subsets of BL images
-        this->drawRow(context, canvas, true, true, true);
+        this->drawRow(canvas, true, true, true);
 
         canvas->restore();
 
@@ -289,8 +305,9 @@ private:
 
 private:
     SkTArray<sk_sp<SkImage>> fLabels;
+    sk_sp<SkImage> fReferenceImages[2];
 
-    typedef GM INHERITED;
+    using INHERITED = GM;
 };
 
 DEF_GM(return new FlippityGM;)
