@@ -9,11 +9,14 @@
 #include "include/private/SkMacros.h"
 #include "src/core/SkArenaAlloc.h"
 #include "src/core/SkBlendModePriv.h"
+#include "src/core/SkColorFilterBase.h"
 #include "src/core/SkColorSpacePriv.h"
 #include "src/core/SkColorSpaceXformSteps.h"
 #include "src/core/SkCoreBlitters.h"
 #include "src/core/SkLRUCache.h"
+#include "src/core/SkMatrixProvider.h"
 #include "src/core/SkOpts.h"
+#include "src/core/SkPaintPriv.h"
 #include "src/core/SkVM.h"
 #include "src/shaders/SkColorFilterShader.h"
 
@@ -26,21 +29,21 @@ namespace {
     struct BlitterUniforms {
         int       right;  // First device x + blit run length n, used to get device x coordinate.
         int       y;      // Device y coordinate.
-        SkColor4f paint;  // In device color space.
     };
     static_assert(SkIsAlign4(sizeof(BlitterUniforms)), "");
     static constexpr int kBlitterUniformsCount = sizeof(BlitterUniforms) / 4;
 
-    enum class Coverage { Full, UniformA8, MaskA8, MaskLCD16, Mask3D };
+    enum class Coverage { Full, UniformF, MaskA8, MaskLCD16, Mask3D };
 
     struct Params {
-        sk_sp<SkShader> shader;
-        sk_sp<SkShader> clip;
-        SkColorInfo     dst;
-        SkBlendMode     blendMode;
-        Coverage        coverage;
-        SkFilterQuality quality;
-        SkMatrix        ctm;
+        sk_sp<SkShader>         shader;
+        sk_sp<SkShader>         clip;
+        SkColorInfo             dst;
+        SkBlendMode             blendMode;
+        Coverage                coverage;
+        SkColor4f               paint;
+        SkFilterQuality         quality;
+        const SkMatrixProvider& matrices;
 
         Params withCoverage(Coverage c) const {
             Params p = *this;
@@ -59,7 +62,7 @@ namespace {
                  blendMode,
                  coverage;
         uint32_t padding{0};
-        // Params::quality and Params::ctm are only passed to {shader,clip}->program(),
+        // Params::{paint,quality,matrices} are only passed to {shader,clip}->program(),
         // not used here by the blitter itself.  No need to include them in the key;
         // they'll be folded into the shader key if used.
 
@@ -107,30 +110,40 @@ namespace {
 
     static void release_program_cache() { }
 
+    static skvm::Coord device_coord(skvm::Builder* p, skvm::Uniforms* uniforms) {
+        skvm::I32 dx = p->uniform32(uniforms->base, offsetof(BlitterUniforms, right))
+                     - p->index(),
+                  dy = p->uniform32(uniforms->base, offsetof(BlitterUniforms, y));
+        return {
+            to_F32(dx) + 0.5f,
+            to_F32(dy) + 0.5f,
+        };
+    }
+
     // If build_program() can't build this program, cache_key() sets *ok to false.
     static Key cache_key(const Params& params,
                          skvm::Uniforms* uniforms, SkArenaAlloc* alloc, bool* ok) {
+        // Take care to match build_program()'s reuse of the paint color uniforms.
+        skvm::Uniform r = uniforms->pushF(params.paint.fR),
+                      g = uniforms->pushF(params.paint.fG),
+                      b = uniforms->pushF(params.paint.fB),
+                      a = uniforms->pushF(params.paint.fA);
         auto hash_shader = [&](const sk_sp<SkShader>& shader) {
             const SkShaderBase* sb = as_SB(shader);
             skvm::Builder p;
 
-            skvm::I32 dx = p.uniform32(uniforms->base, offsetof(BlitterUniforms, right))
-                         - p.index(),
-                      dy = p.uniform32(uniforms->base, offsetof(BlitterUniforms, y));
-            skvm::F32 x = to_f32(dx) + 0.5f,
-                      y = to_f32(dy) + 0.5f;
-
+            skvm::Coord device = device_coord(&p, uniforms);
             skvm::Color paint = {
-                p.uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fR)),
-                p.uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fG)),
-                p.uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fB)),
-                p.uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fA)),
+                p.uniformF(r),
+                p.uniformF(g),
+                p.uniformF(b),
+                p.uniformF(a),
             };
 
             uint64_t hash = 0;
             if (auto c = sb->program(&p,
-                                     x,y, paint,
-                                     params.ctm, /*localM=*/nullptr,
+                                     device,/*local=*/device, paint,
+                                     params.matrices, /*localM=*/nullptr,
                                      params.quality, params.dst,
                                      uniforms,alloc)) {
                 hash = p.hash();
@@ -157,20 +170,6 @@ namespace {
             }
         }
 
-        switch (params.dst.colorType()) {
-            default: *ok = false;
-                     break;
-
-            case kRGB_565_SkColorType:
-            case kRGB_888x_SkColorType:
-            case kRGBA_8888_SkColorType:
-            case kBGRA_8888_SkColorType:
-            case kRGBA_1010102_SkColorType:
-            case kBGRA_1010102_SkColorType:
-            case kRGB_101010x_SkColorType:
-            case kBGR_101010x_SkColorType:  break;
-        }
-
         return {
             shaderHash,
               clipHash,
@@ -186,29 +185,21 @@ namespace {
                               skvm::Uniforms* uniforms, SkArenaAlloc* alloc) {
         // First two arguments are always uniforms and the destination buffer.
         uniforms->base    = p->uniform();
-        skvm::Arg dst_ptr = p->arg(SkColorTypeBytesPerPixel(params.dst.colorType()));
-        // Other arguments depend on params.coverage:
+        skvm::Ptr dst_ptr = p->arg(SkColorTypeBytesPerPixel(params.dst.colorType()));
+        // A SpriteShader (in this file) may next use one argument as its varying source.
+        // Subsequent arguments depend on params.coverage:
         //    - Full:      (no more arguments)
         //    - Mask3D:    mul varying, add varying, 8-bit coverage varying
         //    - MaskA8:    8-bit coverage varying
         //    - MaskLCD16: 565 coverage varying
-        //    - UniformA8: 8-bit coverage uniform
+        //    - UniformF:  float coverage uniform
 
-        skvm::I32 dx = p->uniform32(uniforms->base, offsetof(BlitterUniforms, right))
-                     - p->index(),
-                  dy = p->uniform32(uniforms->base, offsetof(BlitterUniforms, y));
-        skvm::F32 x = to_f32(dx) + 0.5f,
-                  y = to_f32(dy) + 0.5f;
+        skvm::Coord device = device_coord(p, uniforms);
+        skvm::Color paint = p->uniformColor(params.paint, uniforms);
 
-        skvm::Color paint = {
-            p->uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fR)),
-            p->uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fG)),
-            p->uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fB)),
-            p->uniformF(uniforms->base, offsetof(BlitterUniforms, paint.fA)),
-        };
-
-        skvm::Color src = as_SB(params.shader)->program(p, x,y, paint,
-                                                        params.ctm, /*localM=*/nullptr,
+        // See note about arguments above: a SpriteShader will call p->arg() once during program().
+        skvm::Color src = as_SB(params.shader)->program(p, device,/*local=*/device, paint,
+                                                        params.matrices, /*localM=*/nullptr,
                                                         params.quality, params.dst,
                                                         uniforms, alloc);
         SkASSERT(src);
@@ -242,106 +233,66 @@ namespace {
             src_in_gamut = true;
         }
 
-        // There are several orderings here of when we load dst and coverage
-        // and how coverage is applied, and to complicate things, LCD coverage
-        // needs to know dst.a.  We're careful to assert it's loaded in time.
-        skvm::Color dst;
-        SkDEBUGCODE(bool dst_loaded = false;)
-
-        // load_coverage() returns false when there's no need to apply coverage.
-        auto load_coverage = [&](skvm::Color* cov) {
-            bool partial_coverage = true;
-            switch (params.coverage) {
-                case Coverage::Full: cov->r = cov->g = cov->b = cov->a = p->splat(1.0f);
-                                     partial_coverage = false;
-                                     break;
-
-                case Coverage::UniformA8: cov->r = cov->g = cov->b = cov->a =
-                                          from_unorm(8, p->uniform8(p->uniform(), 0));
-                                          break;
-
-                case Coverage::Mask3D:
-                case Coverage::MaskA8: cov->r = cov->g = cov->b = cov->a =
-                                       from_unorm(8, p->load8(p->varying<uint8_t>()));
-                                       break;
-
-                case Coverage::MaskLCD16:
-                    SkASSERT(dst_loaded);
-                    *cov = unpack_565(p->load16(p->varying<uint16_t>()));
-                    cov->a = select(src.a < dst.a, min(cov->r, min(cov->g, cov->b))
-                                                 , max(cov->r, max(cov->g, cov->b)));
-                    break;
-            }
-
-            if (params.clip) {
-                skvm::Color clip = as_SB(params.clip)->program(p, x,y, paint,
-                                                               params.ctm, /*localM=*/nullptr,
-                                                               params.quality, params.dst,
-                                                               uniforms, alloc);
-                SkAssertResult(clip);
-                cov->r *= clip.a;  // We use the alpha channel of clip for all four.
-                cov->g *= clip.a;
-                cov->b *= clip.a;
-                cov->a *= clip.a;
-                return true;
-            }
-
-            return partial_coverage;
-        };
-
-        // The math for some blend modes lets us fold coverage into src before the blend,
-        // obviating the need for the lerp afterwards. This early-coverage strategy tends
-        // to be both faster and require fewer registers.
-        bool lerp_coverage_post_blend = true;
-        if (SkBlendMode_ShouldPreScaleCoverage(params.blendMode,
-                                               params.coverage == Coverage::MaskLCD16)) {
-            skvm::Color cov;
-            if (load_coverage(&cov)) {
-                src.r *= cov.r;
-                src.g *= cov.g;
-                src.b *= cov.b;
-                src.a *= cov.a;
-            }
-            lerp_coverage_post_blend = false;
-        }
-
-        // Load up the destination color.
-        SkDEBUGCODE(dst_loaded = true;)
-        switch (params.dst.colorType()) {
-            default: SkUNREACHABLE;
-            case kRGB_565_SkColorType: dst = unpack_565(p->load16(dst_ptr));
-                                       break;
-
-            case  kRGB_888x_SkColorType: [[fallthrough]];
-            case kRGBA_8888_SkColorType: dst = unpack_8888(p->load32(dst_ptr));
-                                         break;
-
-            case kBGRA_8888_SkColorType: dst = unpack_8888(p->load32(dst_ptr));
-                                         std::swap(dst.r, dst.b);
-                                         break;
-
-            case  kRGB_101010x_SkColorType: [[fallthrough]];
-            case kRGBA_1010102_SkColorType: dst = unpack_1010102(p->load32(dst_ptr));
-                                            break;
-
-            case  kBGR_101010x_SkColorType: [[fallthrough]];
-            case kBGRA_1010102_SkColorType: dst = unpack_1010102(p->load32(dst_ptr));
-                                            std::swap(dst.r, dst.b);
-                                            break;
-        }
-
-        // When a destination is known opaque, we may assume it both starts and stays fully
-        // opaque, ignoring any math that disagrees.  This sometimes trims a little work.
+        // Load the destination color.
+        skvm::PixelFormat dstFormat = skvm::SkColorType_to_PixelFormat(params.dst.colorType());
+        skvm::Color dst = p->load(dstFormat, dst_ptr);
         if (params.dst.isOpaque()) {
+            // When a destination is known opaque, we may assume it both starts and stays fully
+            // opaque, ignoring any math that disagrees.  This sometimes trims a little work.
             dst.a = p->splat(1.0f);
         } else if (params.dst.alphaType() == kUnpremul_SkAlphaType) {
+            // All our blending works in terms of premul.
             dst = premul(dst);
         }
 
-        src = blend(params.blendMode, src, dst);
+        // Load coverage.
+        skvm::Color cov;
+        switch (params.coverage) {
+            case Coverage::Full:
+                cov.r = cov.g = cov.b = cov.a = p->splat(1.0f);
+                break;
+
+            case Coverage::UniformF:
+                cov.r = cov.g = cov.b = cov.a = p->uniformF(p->uniform(), 0);
+                break;
+
+            case Coverage::Mask3D:
+            case Coverage::MaskA8:
+                cov.r = cov.g = cov.b = cov.a = from_unorm(8, p->load8(p->varying<uint8_t>()));
+                break;
+
+            case Coverage::MaskLCD16: {
+                skvm::PixelFormat fmt = skvm::SkColorType_to_PixelFormat(kRGB_565_SkColorType);
+                cov = p->load(fmt, p->varying<uint16_t>());
+                cov.a = select(src.a < dst.a, min(cov.r, min(cov.g, cov.b))
+                                            , max(cov.r, max(cov.g, cov.b)));
+            } break;
+        }
+        if (params.clip) {
+            skvm::Color clip = as_SB(params.clip)->program(p, device,/*local=*/device, paint,
+                                                           params.matrices, /*localM=*/nullptr,
+                                                           params.quality, params.dst,
+                                                           uniforms, alloc);
+            SkAssertResult(clip);
+            cov.r *= clip.a;  // We use the alpha channel of clip for all four.
+            cov.g *= clip.a;
+            cov.b *= clip.a;
+            cov.a *= clip.a;
+        }
+
+        // The math for some blend modes lets us fold coverage into src before the blend,
+        // which is simpler than the canonical post-blend lerp().
+        if (SkBlendMode_ShouldPreScaleCoverage(params.blendMode,
+                                               params.coverage == Coverage::MaskLCD16)) {
+            src.r *= cov.r;
+            src.g *= cov.g;
+            src.b *= cov.b;
+            src.a *= cov.a;
+
+            src = blend(params.blendMode, src, dst);
+        } else {
+            src = blend(params.blendMode, src, dst);
 
-        // Lerp with coverage post-blend if needed.
-        if (skvm::Color cov; lerp_coverage_post_blend && load_coverage(&cov)) {
             src.r = lerp(dst.r, src.r, cov.r);
             src.g = lerp(dst.g, src.g, cov.g);
             src.b = lerp(dst.b, src.b, cov.b);
@@ -349,6 +300,7 @@ namespace {
         }
 
         if (params.dst.isOpaque()) {
+            // (See the note above when loading the destination color.)
             src.a = p->splat(1.0f);
         } else if (params.dst.alphaType() == kUnpremul_SkAlphaType) {
             src = unpremul(src);
@@ -359,52 +311,22 @@ namespace {
             // An in-gamut src blended with an in-gamut dst should stay in gamut.
             // Being in-gamut implies all channels are in [0,1], so no need to clamp.
             // We allow one ulp error above 1.0f, and about that much (~1.2e-7) below 0.
-            skvm::F32 lo = bit_cast(p->splat(0xb400'0000)),
-                      hi = bit_cast(p->splat(0x3f80'0001));
+            skvm::F32 lo = pun_to_F32(p->splat(0xb400'0000)),
+                      hi = pun_to_F32(p->splat(0x3f80'0001));
             assert_true(src.r == clamp(src.r, lo, hi), src.r);
             assert_true(src.g == clamp(src.g, lo, hi), src.g);
             assert_true(src.b == clamp(src.b, lo, hi), src.b);
             assert_true(src.a == clamp(src.a, lo, hi), src.a);
         } else if (SkColorTypeIsNormalized(params.dst.colorType())) {
-            src.r = clamp01(src.r);
-            src.g = clamp01(src.g);
-            src.b = clamp01(src.b);
-            src.a = clamp01(src.a);
+            src = clamp01(src);
         }
 
-        // Store back to the destination.
-        switch (params.dst.colorType()) {
-            default: SkUNREACHABLE;
-
-            case kRGB_565_SkColorType:
-                store16(dst_ptr, pack(pack(to_unorm(5,src.b),
-                                           to_unorm(6,src.g), 5),
-                                           to_unorm(5,src.r),11));
-                break;
-
-            case kBGRA_8888_SkColorType: std::swap(src.r, src.b);  [[fallthrough]];
-            case  kRGB_888x_SkColorType:                           [[fallthrough]];
-            case kRGBA_8888_SkColorType:
-                 store32(dst_ptr, pack(pack(to_unorm(8, src.r),
-                                            to_unorm(8, src.g), 8),
-                                       pack(to_unorm(8, src.b),
-                                            to_unorm(8, src.a), 8), 16));
-                 break;
-
-            case  kBGR_101010x_SkColorType:                          [[fallthrough]];
-            case kBGRA_1010102_SkColorType: std::swap(src.r, src.b); [[fallthrough]];
-            case  kRGB_101010x_SkColorType:                          [[fallthrough]];
-            case kRGBA_1010102_SkColorType:
-                 store32(dst_ptr, pack(pack(to_unorm(10, src.r),
-                                            to_unorm(10, src.g), 10),
-                                       pack(to_unorm(10, src.b),
-                                            to_unorm( 2, src.a), 10), 20));
-                 break;
-        }
+        // Write it out!
+        store(dstFormat, dst_ptr, src);
     }
 
 
-    struct NoopColorFilter : public SkColorFilter {
+    struct NoopColorFilter : public SkColorFilterBase {
         skvm::Color onProgram(skvm::Builder*, skvm::Color c,
                               SkColorSpace*, skvm::Uniforms*, SkArenaAlloc*) const override {
             return c;
@@ -417,6 +339,32 @@ namespace {
         const char* getTypeName() const override { return "NoopColorFilter"; }
     };
 
+    struct SpriteShader : public SkShaderBase {
+        explicit SpriteShader(SkPixmap sprite) : fSprite(sprite) {}
+
+        SkPixmap fSprite;
+
+        // Only created here temporarily... never serialized.
+        Factory      getFactory() const override { return nullptr; }
+        const char* getTypeName() const override { return "SpriteShader"; }
+
+        bool isOpaque() const override { return fSprite.isOpaque(); }
+
+        skvm::Color onProgram(skvm::Builder* p,
+                              skvm::Coord /*device*/, skvm::Coord /*local*/, skvm::Color /*paint*/,
+                              const SkMatrixProvider&, const SkMatrix* /*localM*/,
+                              SkFilterQuality, const SkColorInfo& dst,
+                              skvm::Uniforms* uniforms, SkArenaAlloc*) const override {
+            const SkColorType ct = fSprite.colorType();
+
+            skvm::PixelFormat fmt = skvm::SkColorType_to_PixelFormat(ct);
+
+            skvm::Color c = p->load(fmt, p->arg(SkColorTypeBytesPerPixel(ct)));
+
+            return SkColorSpaceXformSteps{fSprite, dst}.program(p, uniforms, c);
+        }
+    };
+
     struct DitherShader : public SkShaderBase {
         explicit DitherShader(sk_sp<SkShader> shader) : fShader(std::move(shader)) {}
 
@@ -428,13 +376,14 @@ namespace {
 
         bool isOpaque() const override { return fShader->isOpaque(); }
 
-        skvm::Color onProgram(skvm::Builder* p, skvm::F32 x, skvm::F32 y, skvm::Color paint,
-                              const SkMatrix& ctm, const SkMatrix* localM,
+        skvm::Color onProgram(skvm::Builder* p,
+                              skvm::Coord device, skvm::Coord local, skvm::Color paint,
+                              const SkMatrixProvider& matrices, const SkMatrix* localM,
                               SkFilterQuality quality, const SkColorInfo& dst,
                               skvm::Uniforms* uniforms, SkArenaAlloc* alloc) const override {
             // Run our wrapped shader.
-            skvm::Color c = as_SB(fShader)->program(p, x,y, paint,
-                                                    ctm,localM, quality,dst, uniforms,alloc);
+            skvm::Color c = as_SB(fShader)->program(p, device,local, paint,
+                                                    matrices,localM, quality,dst, uniforms,alloc);
             if (!c) {
                 return {};
             }
@@ -467,8 +416,10 @@ namespace {
 
             // See SkRasterPipeline dither stage.
             // This is 8x8 ordered dithering.  From here we'll only need dx and dx^dy.
-            skvm::I32 X =     trunc(x - 0.5f),
-                      Y = X ^ trunc(y - 0.5f);
+            SkASSERT(local.x.id == device.x.id);
+            SkASSERT(local.y.id == device.y.id);
+            skvm::I32 X =     trunc(device.x - 0.5f),
+                      Y = X ^ trunc(device.y - 0.5f);
 
             // If X's low bits are abc and Y's def, M is fcebda,
             // 6 bits producing all values [0,63] shuffled over an 8x8 grid.
@@ -487,7 +438,7 @@ namespace {
             // we can bake it in without hurting the cache hit rate.
             float scale = rate * (  2/128.0f),
                   bias  = rate * (-63/128.0f);
-            skvm::F32 dither = to_f32(M) * scale + bias;
+            skvm::F32 dither = to_F32(M) * scale + bias;
             c.r += dither;
             c.g += dither;
             c.b += dither;
@@ -499,18 +450,52 @@ namespace {
         }
     };
 
+    // This is similar to using SkShaders::Color(paint.getColor4f(), nullptr),
+    // but uses the blitter-provided paint color uniforms instead of pushing its own.
+    struct PaintColorShader : public SkShaderBase {
+        explicit PaintColorShader(bool isOpaque) : fIsOpaque(isOpaque) {}
+
+        const bool fIsOpaque;
+
+        // Only created here temporarily... never serialized.
+        Factory      getFactory() const override { return nullptr; }
+        const char* getTypeName() const override { return "PaintColorShader"; }
+
+        bool isOpaque() const override { return fIsOpaque; }
+
+        skvm::Color onProgram(skvm::Builder*,
+                              skvm::Coord, skvm::Coord, skvm::Color paint,
+                              const SkMatrixProvider&, const SkMatrix*,
+                              SkFilterQuality, const SkColorInfo&,
+                              skvm::Uniforms*, SkArenaAlloc*) const override {
+            // Incoming `paint` is unpremul in the destination color space,
+            // so we just need to premul it.
+            return premul(paint);
+        }
+    };
+
     static Params effective_params(const SkPixmap& device,
-                                   const SkPaint& paint,
-                                   const SkMatrix& ctm,
+                                   const SkPixmap* sprite,
+                                   SkPaint paint,
+                                   const SkMatrixProvider& matrices,
                                    sk_sp<SkShader> clip) {
-        // Color filters have been handled for us by SkBlitter::Choose().
+        // Sprites take priority over any shader.  (There's rarely one set, and it's meaningless.)
+        if (sprite) {
+            paint.setShader(sk_make_sp<SpriteShader>(*sprite));
+        }
+
+        // Normal blitters will have already folded color filters into their shader,
+        // but we may still need to do that here for SpriteShaders.
+        if (paint.getColorFilter()) {
+            SkPaintPriv::RemoveColorFilter(&paint, device.colorSpace());
+        }
         SkASSERT(!paint.getColorFilter());
 
         // If there's no explicit shader, the paint color is the shader,
         // but if there is a shader, it's modulated by the paint alpha.
         sk_sp<SkShader> shader = paint.refShader();
         if (!shader) {
-            shader = SkShaders::Color(paint.getColor4f(), nullptr);
+            shader = sk_make_sp<PaintColorShader>(paint.getColor4f().isOpaque());
         } else if (paint.getAlphaf() < 1.0f) {
             shader = sk_make_sp<SkColorFilterShader>(std::move(shader),
                                                      paint.getAlphaf(),
@@ -540,35 +525,39 @@ namespace {
             blendMode =  SkBlendMode::kSrc;
         }
 
+        SkColor4f paintColor = paint.getColor4f();
+        SkColorSpaceXformSteps{sk_srgb_singleton(), kUnpremul_SkAlphaType,
+                               device.colorSpace(), kUnpremul_SkAlphaType}
+            .apply(paintColor.vec());
+
         return {
             std::move(shader),
             std::move(clip),
             { device.colorType(), device.alphaType(), device.refColorSpace() },
             blendMode,
             Coverage::Full,  // Placeholder... withCoverage() will change as needed.
+            paintColor,
             paint.getFilterQuality(),
-            ctm,
+            matrices,
         };
     }
 
     class Blitter final : public SkBlitter {
     public:
-        Blitter(const SkPixmap& device,
-                const SkPaint& paint,
-                const SkMatrix& ctm,
-                sk_sp<SkShader> clip,
+        Blitter(const SkPixmap&         device,
+                const SkPaint&          paint,
+                const SkPixmap*         sprite,
+                SkIPoint                spriteOffset,
+                const SkMatrixProvider& matrices,
+                sk_sp<SkShader>         clip,
                 bool* ok)
             : fDevice(device)
-            , fUniforms(kBlitterUniformsCount)
-            , fParams(effective_params(device, paint, ctm, std::move(clip)))
+            , fSprite(sprite ? *sprite : SkPixmap{})
+            , fSpriteOffset(spriteOffset)
+            , fUniforms(skvm::Ptr{0}, kBlitterUniformsCount)
+            , fParams(effective_params(device, sprite, paint, matrices, std::move(clip)))
             , fKey(cache_key(fParams, &fUniforms, &fAlloc, ok))
-            , fPaint([&]{
-                SkColor4f color = paint.getColor4f();
-                SkColorSpaceXformSteps{sk_srgb_singleton(), kUnpremul_SkAlphaType,
-                                       device.colorSpace(), kUnpremul_SkAlphaType}
-                    .apply(color.vec());
-                return color;
-            }()) {}
+        {}
 
         ~Blitter() override {
             if (SkLRUCache<Key, skvm::Program>* cache = try_acquire_program_cache()) {
@@ -583,7 +572,7 @@ namespace {
                     }
                 };
                 cache_program(std::move(fBlitH),         Coverage::Full);
-                cache_program(std::move(fBlitAntiH),     Coverage::UniformA8);
+                cache_program(std::move(fBlitAntiH),     Coverage::UniformF);
                 cache_program(std::move(fBlitMaskA8),    Coverage::MaskA8);
                 cache_program(std::move(fBlitMask3D),    Coverage::Mask3D);
                 cache_program(std::move(fBlitMaskLCD16), Coverage::MaskLCD16);
@@ -594,11 +583,12 @@ namespace {
 
     private:
         SkPixmap        fDevice;
+        const SkPixmap  fSprite;                  // See isSprite().
+        const SkIPoint  fSpriteOffset;
         skvm::Uniforms  fUniforms;                // Most data is copied directly into fUniforms,
         SkArenaAlloc    fAlloc{2*sizeof(void*)};  // but a few effects need to ref large content.
         const Params    fParams;
         const Key       fKey;
-        const SkColor4f fPaint;
         skvm::Program   fBlitH,
                         fBlitAntiH,
                         fBlitMaskA8,
@@ -639,10 +629,6 @@ namespace {
                     builder.dump();
                     program.dump();
 
-                    SkString path = SkStringPrintf("/tmp/%s.dot", debug_name(key).c_str());
-                    SkFILEWStream tmp(path.c_str());
-                    builder.dot(&tmp);
-
                     missed++;
                 }
                 if (0 == total++) {
@@ -654,26 +640,42 @@ namespace {
         }
 
         void updateUniforms(int right, int y) {
-            BlitterUniforms uniforms{right, y, fPaint};
+            BlitterUniforms uniforms{right, y};
             memcpy(fUniforms.buf.data(), &uniforms, sizeof(BlitterUniforms));
         }
 
+        const void* isSprite(int x, int y) const {
+            if (fSprite.colorType() != kUnknown_SkColorType) {
+                return fSprite.addr(x - fSpriteOffset.x(),
+                                    y - fSpriteOffset.y());
+            }
+            return nullptr;
+        }
+
         void blitH(int x, int y, int w) override {
             if (fBlitH.empty()) {
                 fBlitH = this->buildProgram(Coverage::Full);
             }
             this->updateUniforms(x+w, y);
-            fBlitH.eval(w, fUniforms.buf.data(), fDevice.addr(x,y));
+            if (const void* sprite = this->isSprite(x,y)) {
+                fBlitH.eval(w, fUniforms.buf.data(), fDevice.addr(x,y), sprite);
+            } else {
+                fBlitH.eval(w, fUniforms.buf.data(), fDevice.addr(x,y));
+            }
         }
 
         void blitAntiH(int x, int y, const SkAlpha cov[], const int16_t runs[]) override {
             if (fBlitAntiH.empty()) {
-                fBlitAntiH = this->buildProgram(Coverage::UniformA8);
+                fBlitAntiH = this->buildProgram(Coverage::UniformF);
             }
             for (int16_t run = *runs; run > 0; run = *runs) {
                 this->updateUniforms(x+run, y);
-                fBlitAntiH.eval(run, fUniforms.buf.data(), fDevice.addr(x,y), cov);
-
+                const float covF = *cov * (1/255.0f);
+                if (const void* sprite = this->isSprite(x,y)) {
+                    fBlitAntiH.eval(run, fUniforms.buf.data(), fDevice.addr(x,y), sprite, &covF);
+                } else {
+                    fBlitAntiH.eval(run, fUniforms.buf.data(), fDevice.addr(x,y), &covF);
+                }
                 x    += run;
                 runs += run;
                 cov  += run;
@@ -722,11 +724,21 @@ namespace {
 
                     if (program == &fBlitMask3D) {
                         size_t plane = mask.computeImageSize();
-                        program->eval(w, fUniforms.buf.data(), dptr, mptr + 1*plane
-                                                                   , mptr + 2*plane
-                                                                   , mptr + 0*plane);
+                        if (const void* sprite = this->isSprite(x,y)) {
+                            program->eval(w, fUniforms.buf.data(), dptr, sprite, mptr + 1*plane
+                                                                               , mptr + 2*plane
+                                                                               , mptr + 0*plane);
+                        } else {
+                            program->eval(w, fUniforms.buf.data(), dptr, mptr + 1*plane
+                                                                       , mptr + 2*plane
+                                                                       , mptr + 0*plane);
+                        }
                     } else {
-                        program->eval(w, fUniforms.buf.data(), dptr, mptr);
+                        if (const void* sprite = this->isSprite(x,y)) {
+                            program->eval(w, fUniforms.buf.data(), dptr, sprite, mptr);
+                        } else {
+                            program->eval(w, fUniforms.buf.data(), dptr, mptr);
+                        }
                     }
                 }
             }
@@ -737,10 +749,27 @@ namespace {
 
 SkBlitter* SkCreateSkVMBlitter(const SkPixmap& device,
                                const SkPaint& paint,
-                               const SkMatrix& ctm,
+                               const SkMatrixProvider& matrices,
                                SkArenaAlloc* alloc,
                                sk_sp<SkShader> clip) {
     bool ok = true;
-    auto blitter = alloc->make<Blitter>(device, paint, ctm, std::move(clip), &ok);
+    auto blitter = alloc->make<Blitter>(device, paint, /*sprite=*/nullptr, SkIPoint{0,0},
+                                        matrices, std::move(clip), &ok);
+    return ok ? blitter : nullptr;
+}
+
+SkBlitter* SkCreateSkVMSpriteBlitter(const SkPixmap& device,
+                                     const SkPaint& paint,
+                                     const SkPixmap& sprite,
+                                     int left, int top,
+                                     SkArenaAlloc* alloc,
+                                     sk_sp<SkShader> clip) {
+    if (paint.getMaskFilter()) {
+        // TODO: SkVM support for mask filters?  definitely possible!
+        return nullptr;
+    }
+    bool ok = true;
+    auto blitter = alloc->make<Blitter>(device, paint, &sprite, SkIPoint{left,top},
+                                        SkSimpleMatrixProvider{SkMatrix{}}, std::move(clip), &ok);
     return ok ? blitter : nullptr;
 }
