@@ -10,62 +10,49 @@
 
 #include "include/gpu/GrBackendSemaphore.h"
 #include "include/gpu/GrBackendSurface.h"
-#include "include/gpu/GrContext.h"
+#include "include/gpu/GrDirectContext.h"
 #include "src/core/SkCompressedDataUtils.h"
 #include "src/core/SkMathPriv.h"
-#include "src/core/SkMipMap.h"
+#include "src/core/SkMipmap.h"
+#include "src/gpu/GrAttachment.h"
 #include "src/gpu/GrAuditTrail.h"
+#include "src/gpu/GrBackendUtils.h"
 #include "src/gpu/GrCaps.h"
-#include "src/gpu/GrContextPriv.h"
 #include "src/gpu/GrDataUtils.h"
+#include "src/gpu/GrDirectContextPriv.h"
 #include "src/gpu/GrGpuResourcePriv.h"
 #include "src/gpu/GrNativeRect.h"
 #include "src/gpu/GrPathRendering.h"
 #include "src/gpu/GrPipeline.h"
-#include "src/gpu/GrRenderTargetPriv.h"
+#include "src/gpu/GrRenderTarget.h"
 #include "src/gpu/GrResourceCache.h"
 #include "src/gpu/GrResourceProvider.h"
+#include "src/gpu/GrRingBuffer.h"
 #include "src/gpu/GrSemaphore.h"
-#include "src/gpu/GrStagingBuffer.h"
-#include "src/gpu/GrStencilAttachment.h"
+#include "src/gpu/GrStagingBufferManager.h"
 #include "src/gpu/GrStencilSettings.h"
-#include "src/gpu/GrSurfacePriv.h"
-#include "src/gpu/GrTexturePriv.h"
 #include "src/gpu/GrTextureProxyPriv.h"
 #include "src/gpu/GrTracing.h"
+#include "src/sksl/SkSLCompiler.h"
 #include "src/utils/SkJSONWriter.h"
 
-static const size_t kMinStagingBufferSize = 32 * 1024;
-
 ////////////////////////////////////////////////////////////////////////////////
 
-GrGpu::GrGpu(GrContext* context) : fResetBits(kAll_GrBackendState), fContext(context) {}
+GrGpu::GrGpu(GrDirectContext* direct) : fResetBits(kAll_GrBackendState), fContext(direct) {}
 
 GrGpu::~GrGpu() {
-    SkASSERT(fBusyStagingBuffers.isEmpty());
+    this->callSubmittedProcs(false);
 }
 
-void GrGpu::disconnect(DisconnectType type) {
-    if (DisconnectType::kAbandon == type) {
-        fAvailableStagingBuffers.reset();
-        fActiveStagingBuffers.reset();
-        fBusyStagingBuffers.reset();
-    }
-    fStagingBuffers.clear();
+void GrGpu::initCapsAndCompiler(sk_sp<const GrCaps> caps) {
+    fCaps = std::move(caps);
+    fCompiler = std::make_unique<SkSL::Compiler>(fCaps->shaderCaps());
 }
 
+void GrGpu::disconnect(DisconnectType type) {}
+
 ////////////////////////////////////////////////////////////////////////////////
 
-bool GrGpu::IsACopyNeededForMips(const GrCaps* caps, const GrTextureProxy* texProxy,
-                                 GrSamplerState::Filter filter) {
-    SkASSERT(texProxy);
-    if (filter != GrSamplerState::Filter::kMipMap || texProxy->mipMapped() == GrMipMapped::kYes ||
-        !caps->mipMapSupport()) {
-        return false;
-    }
-    return SkMipMap::ComputeLevelCount(texProxy->width(), texProxy->height()) > 0;
-}
-
 static bool validate_texel_levels(SkISize dimensions, GrColorType texelColorType,
                                   const GrMipLevel* texels, int mipLevelCount, const GrCaps* caps) {
     SkASSERT(mipLevelCount > 0);
@@ -124,7 +111,7 @@ sk_sp<GrTexture> GrGpu::createTextureCommon(SkISize dimensions,
         return nullptr;
     }
 
-    GrMipMapped mipMapped = mipLevelCount > 1 ? GrMipMapped::kYes : GrMipMapped::kNo;
+    GrMipmapped mipMapped = mipLevelCount > 1 ? GrMipmapped::kYes : GrMipmapped::kNo;
     if (!this->caps()->validateSurfaceParams(dimensions, format, renderable, renderTargetSampleCnt,
                                              mipMapped)) {
         return nullptr;
@@ -164,11 +151,11 @@ sk_sp<GrTexture> GrGpu::createTexture(SkISize dimensions,
                                       const GrBackendFormat& format,
                                       GrRenderable renderable,
                                       int renderTargetSampleCnt,
-                                      GrMipMapped mipMapped,
+                                      GrMipmapped mipMapped,
                                       SkBudgeted budgeted,
                                       GrProtected isProtected) {
     int mipLevelCount = 1;
-    if (mipMapped == GrMipMapped::kYes) {
+    if (mipMapped == GrMipmapped::kYes) {
         mipLevelCount =
                 32 - SkCLZ(static_cast<uint32_t>(std::max(dimensions.fWidth, dimensions.fHeight)));
     }
@@ -176,8 +163,8 @@ sk_sp<GrTexture> GrGpu::createTexture(SkISize dimensions,
             this->caps()->shouldInitializeTextures() ? (1 << mipLevelCount) - 1 : 0;
     auto tex = this->createTextureCommon(dimensions, format, renderable, renderTargetSampleCnt,
                                          budgeted, isProtected, mipLevelCount, levelClearMask);
-    if (tex && mipMapped == GrMipMapped::kYes && levelClearMask) {
-        tex->texturePriv().markMipMapsClean();
+    if (tex && mipMapped == GrMipmapped::kYes && levelClearMask) {
+        tex->markMipmapsClean();
     }
     return tex;
 }
@@ -233,7 +220,7 @@ sk_sp<GrTexture> GrGpu::createTexture(SkISize dimensions,
             markMipLevelsClean = true;
         }
         if (markMipLevelsClean) {
-            tex->texturePriv().markMipMapsClean();
+            tex->markMipmapsClean();
         }
     }
     return tex;
@@ -242,7 +229,7 @@ sk_sp<GrTexture> GrGpu::createTexture(SkISize dimensions,
 sk_sp<GrTexture> GrGpu::createCompressedTexture(SkISize dimensions,
                                                 const GrBackendFormat& format,
                                                 SkBudgeted budgeted,
-                                                GrMipMapped mipMapped,
+                                                GrMipmapped mipMapped,
                                                 GrProtected isProtected,
                                                 const void* data,
                                                 size_t dataSize) {
@@ -261,10 +248,10 @@ sk_sp<GrTexture> GrGpu::createCompressedTexture(SkISize dimensions,
     }
 
     // TODO: expand CompressedDataIsCorrect to work here too
-    SkImage::CompressionType compressionType = this->caps()->compressionType(format);
+    SkImage::CompressionType compressionType = GrBackendFormatToCompressionType(format);
 
     if (dataSize < SkCompressedDataSize(compressionType, dimensions, nullptr,
-                                        mipMapped == GrMipMapped::kYes)) {
+                                        mipMapped == GrMipmapped::kYes)) {
         return nullptr;
     }
     return this->onCreateCompressedTexture(dimensions, format, budgeted, mipMapped, isProtected,
@@ -356,28 +343,6 @@ sk_sp<GrRenderTarget> GrGpu::wrapBackendRenderTarget(const GrBackendRenderTarget
     return rt;
 }
 
-sk_sp<GrRenderTarget> GrGpu::wrapBackendTextureAsRenderTarget(const GrBackendTexture& backendTex,
-                                                              int sampleCnt) {
-    this->handleDirtyContext();
-
-    const GrCaps* caps = this->caps();
-
-    int maxSize = caps->maxTextureSize();
-    if (backendTex.width() > maxSize || backendTex.height() > maxSize) {
-        return nullptr;
-    }
-
-    if (!caps->isFormatRenderable(backendTex.getBackendFormat(), sampleCnt)) {
-        return nullptr;
-    }
-
-    auto rt = this->onWrapBackendTextureAsRenderTarget(backendTex, sampleCnt);
-    if (rt && sampleCnt > 1 && !this->caps()->msaaResolvesAutomatically()) {
-        rt->setRequiresManualMSAAResolve();
-    }
-    return rt;
-}
-
 sk_sp<GrRenderTarget> GrGpu::wrapVulkanSecondaryCBAsRenderTarget(const SkImageInfo& imageInfo,
                                                                  const GrVkDrawableInfo& vkInfo) {
     return this->onWrapVulkanSecondaryCBAsRenderTarget(imageInfo, vkInfo);
@@ -453,7 +418,8 @@ bool GrGpu::writePixels(GrSurface* surface, int left, int top, int width, int he
                         GrColorType surfaceColorType, GrColorType srcColorType,
                         const GrMipLevel texels[], int mipLevelCount, bool prepForTexSampling) {
     TRACE_EVENT0("skia.gpu", TRACE_FUNC);
-    ATRACE_ANDROID_FRAMEWORK_ALWAYS("texture_upload");
+    ATRACE_ANDROID_FRAMEWORK_ALWAYS("Texture upload(%u) %ix%i",
+                                    surface->uniqueID().asUInt(), width, height);
     SkASSERT(surface);
     SkASSERT(!surface->framebufferOnly());
 
@@ -493,7 +459,7 @@ bool GrGpu::writePixels(GrSurface* surface, int left, int top, int width, int he
 
 bool GrGpu::transferPixelsTo(GrTexture* texture, int left, int top, int width, int height,
                              GrColorType textureColorType, GrColorType bufferColorType,
-                             GrGpuBuffer* transferBuffer, size_t offset, size_t rowBytes) {
+                             sk_sp<GrGpuBuffer> transferBuffer, size_t offset, size_t rowBytes) {
     TRACE_EVENT0("skia.gpu", TRACE_FUNC);
     SkASSERT(texture);
     SkASSERT(transferBuffer);
@@ -525,7 +491,7 @@ bool GrGpu::transferPixelsTo(GrTexture* texture, int left, int top, int width, i
 
     this->handleDirtyContext();
     if (this->onTransferPixelsTo(texture, left, top, width, height, textureColorType,
-                                 bufferColorType, transferBuffer, offset, rowBytes)) {
+                                 bufferColorType, std::move(transferBuffer), offset, rowBytes)) {
         SkIRect rect = SkIRect::MakeXYWH(left, top, width, height);
         this->didWriteToSurface(texture, kTopLeft_GrSurfaceOrigin, &rect);
         fStats.incTransfersToTexture();
@@ -537,7 +503,7 @@ bool GrGpu::transferPixelsTo(GrTexture* texture, int left, int top, int width, i
 
 bool GrGpu::transferPixelsFrom(GrSurface* surface, int left, int top, int width, int height,
                                GrColorType surfaceColorType, GrColorType bufferColorType,
-                               GrGpuBuffer* transferBuffer, size_t offset) {
+                               sk_sp<GrGpuBuffer> transferBuffer, size_t offset) {
     TRACE_EVENT0("skia.gpu", TRACE_FUNC);
     SkASSERT(surface);
     SkASSERT(transferBuffer);
@@ -559,7 +525,7 @@ bool GrGpu::transferPixelsFrom(GrSurface* surface, int left, int top, int width,
 
     this->handleDirtyContext();
     if (this->onTransferPixelsFrom(surface, left, top, width, height, surfaceColorType,
-                                   bufferColorType, transferBuffer, offset)) {
+                                   bufferColorType, std::move(transferBuffer), offset)) {
         fStats.incTransfersFromSurface();
         return true;
     }
@@ -569,9 +535,9 @@ bool GrGpu::transferPixelsFrom(GrSurface* surface, int left, int top, int width,
 bool GrGpu::regenerateMipMapLevels(GrTexture* texture) {
     TRACE_EVENT0("skia.gpu", TRACE_FUNC);
     SkASSERT(texture);
-    SkASSERT(this->caps()->mipMapSupport());
-    SkASSERT(texture->texturePriv().mipMapped() == GrMipMapped::kYes);
-    if (!texture->texturePriv().mipMapsAreDirty()) {
+    SkASSERT(this->caps()->mipmapSupport());
+    SkASSERT(texture->mipmapped() == GrMipmapped::kYes);
+    if (!texture->mipmapsAreDirty()) {
         // This can happen when the proxy expects mipmaps to be dirty, but they are not dirty on the
         // actual target. This may be caused by things that the drawingManager could not predict,
         // i.e., ops that don't draw anything, aborting a draw for exceptional circumstances, etc.
@@ -582,7 +548,7 @@ bool GrGpu::regenerateMipMapLevels(GrTexture* texture) {
         return false;
     }
     if (this->onRegenerateMipMapLevels(texture)) {
-        texture->texturePriv().markMipMapsClean();
+        texture->markMipmapsClean();
         return true;
     }
     return false;
@@ -593,11 +559,10 @@ void GrGpu::resetTextureBindings() {
     this->onResetTextureBindings();
 }
 
-void GrGpu::resolveRenderTarget(GrRenderTarget* target, const SkIRect& resolveRect,
-                                ForExternalIO forExternalIO) {
+void GrGpu::resolveRenderTarget(GrRenderTarget* target, const SkIRect& resolveRect) {
     SkASSERT(target);
     this->handleDirtyContext();
-    this->onResolveRenderTarget(target, resolveRect, forExternalIO);
+    this->onResolveRenderTarget(target, resolveRect);
 }
 
 void GrGpu::didWriteToSurface(GrSurface* surface, GrSurfaceOrigin origin, const SkIRect* bounds,
@@ -608,7 +573,7 @@ void GrGpu::didWriteToSurface(GrSurface* surface, GrSurfaceOrigin origin, const
     if (nullptr == bounds || !bounds->isEmpty()) {
         GrTexture* texture = surface->asTexture();
         if (texture && 1 == mipLevels) {
-            texture->texturePriv().markMipMapsDirty();
+            texture->markMipmapsDirty();
         }
     }
 }
@@ -616,48 +581,18 @@ void GrGpu::didWriteToSurface(GrSurface* surface, GrSurfaceOrigin origin, const
 int GrGpu::findOrAssignSamplePatternKey(GrRenderTarget* renderTarget) {
     SkASSERT(this->caps()->sampleLocationsSupport());
     SkASSERT(renderTarget->numSamples() > 1 ||
-             (renderTarget->renderTargetPriv().getStencilAttachment() &&
-              renderTarget->renderTargetPriv().getStencilAttachment()->numSamples() > 1));
+             (renderTarget->getStencilAttachment() &&
+              renderTarget->getStencilAttachment()->numSamples() > 1));
 
     SkSTArray<16, SkPoint> sampleLocations;
     this->querySampleLocations(renderTarget, &sampleLocations);
     return fSamplePatternDictionary.findOrAssignSamplePatternKey(sampleLocations);
 }
 
-#ifdef SK_DEBUG
-bool GrGpu::inStagingBuffers(GrStagingBuffer* b) const {
-    for (const auto& i : fStagingBuffers) {
-        if (b == i.get()) {
-            return true;
-        }
-    }
-    return false;
-}
-
-void GrGpu::validateStagingBuffers() const {
-    for (const auto& i : fStagingBuffers) {
-        GrStagingBuffer* buffer = i.get();
-        SkASSERT(fAvailableStagingBuffers.isInList(buffer) ||
-                 fActiveStagingBuffers.isInList(buffer) ||
-                 fBusyStagingBuffers.isInList(buffer));
-    }
-    for (auto b : fAvailableStagingBuffers) {
-        SkASSERT(this->inStagingBuffers(b));
-    }
-    for (auto b : fActiveStagingBuffers) {
-        SkASSERT(this->inStagingBuffers(b));
-    }
-    for (auto b : fBusyStagingBuffers) {
-        SkASSERT(this->inStagingBuffers(b));
-    }
-}
-#endif
-
-void GrGpu::executeFlushInfo(GrSurfaceProxy* proxies[],
-                             int numProxies,
+void GrGpu::executeFlushInfo(SkSpan<GrSurfaceProxy*> proxies,
                              SkSurface::BackendSurfaceAccess access,
                              const GrFlushInfo& info,
-                             const GrPrepareForExternalIORequests& externalRequests) {
+                             const GrBackendSurfaceMutableState* newState) {
     TRACE_EVENT0("skia.gpu", TRACE_FUNC);
 
     GrResourceProvider* resourceProvider = fContext->priv().resourceProvider();
@@ -671,7 +606,11 @@ void GrGpu::executeFlushInfo(GrSurfaceProxy* proxies[],
                     info.fSignalSemaphores[i],
                     GrResourceProvider::SemaphoreWrapType::kWillSignal,
                     kBorrow_GrWrapOwnership);
-                this->insertSemaphore(semaphores[i].get());
+                // If we failed to wrap the semaphore it means the client didn't give us a valid
+                // semaphore to begin with. Therefore, it is fine to not signal it.
+                if (semaphores[i]) {
+                    this->insertSemaphore(semaphores[i].get());
+                }
             } else {
                 semaphores[i] = resourceProvider->makeSemaphore(false);
                 if (semaphores[i]) {
@@ -685,23 +624,85 @@ void GrGpu::executeFlushInfo(GrSurfaceProxy* proxies[],
     if (info.fFinishedProc) {
         this->addFinishedProc(info.fFinishedProc, info.fFinishedContext);
     }
-    this->prepareSurfacesForBackendAccessAndExternalIO(proxies, numProxies, access,
-                                                       externalRequests);
+
+    if (info.fSubmittedProc) {
+        fSubmittedProcs.emplace_back(info.fSubmittedProc, info.fSubmittedContext);
+    }
+
+    // We currently don't support passing in new surface state for multiple proxies here. The only
+    // time we have multiple proxies is if we are flushing a yuv SkImage which won't have state
+    // updates anyways.
+    SkASSERT(!newState || proxies.count() == 1);
+    SkASSERT(!newState || access == SkSurface::BackendSurfaceAccess::kNoAccess);
+    this->prepareSurfacesForBackendAccessAndStateUpdates(proxies, access, newState);
+}
+
+GrOpsRenderPass* GrGpu::getOpsRenderPass(
+        GrRenderTarget* renderTarget,
+        GrAttachment* stencil,
+        GrSurfaceOrigin origin,
+        const SkIRect& bounds,
+        const GrOpsRenderPass::LoadAndStoreInfo& colorInfo,
+        const GrOpsRenderPass::StencilLoadAndStoreInfo& stencilInfo,
+        const SkTArray<GrSurfaceProxy*, true>& sampledProxies,
+        GrXferBarrierFlags renderPassXferBarriers) {
+#if SK_HISTOGRAMS_ENABLED
+    fCurrentSubmitRenderPassCount++;
+#endif
+    return this->onGetOpsRenderPass(renderTarget, stencil, origin, bounds, colorInfo, stencilInfo,
+                                    sampledProxies, renderPassXferBarriers);
 }
 
 bool GrGpu::submitToGpu(bool syncCpu) {
     this->stats()->incNumSubmitToGpus();
 
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    this->unmapStagingBuffers();
+    if (auto manager = this->stagingBufferManager()) {
+        manager->detachBuffers();
+    }
+
+    if (auto uniformsBuffer = this->uniformsRingBuffer()) {
+        uniformsBuffer->startSubmit(this);
+    }
 
     bool submitted = this->onSubmitToGpu(syncCpu);
 
+    this->callSubmittedProcs(submitted);
+
+    this->reportSubmitHistograms();
+
     return submitted;
 }
 
+void GrGpu::reportSubmitHistograms() {
+#if SK_HISTOGRAMS_ENABLED
+    // The max allowed value for SK_HISTOGRAM_EXACT_LINEAR is 100. If we want to support higher
+    // values we can add SK_HISTOGRAM_CUSTOM_COUNTS but this has a number of buckets that is less
+    // than the number of actual values
+    static constexpr int kMaxRenderPassBucketValue = 100;
+    SK_HISTOGRAM_EXACT_LINEAR("SubmitRenderPasses",
+                              std::min(fCurrentSubmitRenderPassCount, kMaxRenderPassBucketValue),
+                              kMaxRenderPassBucketValue);
+    fCurrentSubmitRenderPassCount = 0;
+#endif
+
+    this->onReportSubmitHistograms();
+}
+
+bool GrGpu::checkAndResetOOMed() {
+    if (fOOMed) {
+        fOOMed = false;
+        return true;
+    }
+    return false;
+}
+
+void GrGpu::callSubmittedProcs(bool success) {
+    for (int i = 0; i < fSubmittedProcs.count(); ++i) {
+        fSubmittedProcs[i].fProc(fSubmittedProcs[i].fContext, success);
+    }
+    fSubmittedProcs.reset();
+}
+
 #ifdef SK_ENABLE_DUMP_GPU
 void GrGpu::dumpJSON(SkJSONWriter* writer) const {
     writer->beginObject();
@@ -740,8 +741,11 @@ void GrGpu::Stats::dump(SkString* out) {
     out->appendf("Transfers to Texture: %d\n", fTransfersToTexture);
     out->appendf("Transfers from Surface: %d\n", fTransfersFromSurface);
     out->appendf("Stencil Buffer Creates: %d\n", fStencilAttachmentCreates);
+    out->appendf("MSAA Attachment Creates: %d\n", fMSAAAttachmentCreates);
     out->appendf("Number of draws: %d\n", fNumDraws);
     out->appendf("Number of Scratch Textures reused %d\n", fNumScratchTexturesReused);
+    out->appendf("Number of Scratch MSAA Attachments reused %d\n",
+                 fNumScratchMSAAAttachmentsReused);
 
     SkASSERT(fNumInlineCompilationFailures == 0);
     out->appendf("Number of Inline compile failures %d\n", fNumInlineCompilationFailures);
@@ -788,11 +792,11 @@ void GrGpu::Stats::dumpKeyValuePairs(SkTArray<SkString>* keys, SkTArray<double>*
 #endif // GR_TEST_UTILS
 
 bool GrGpu::MipMapsAreCorrect(SkISize dimensions,
-                              GrMipMapped mipMapped,
+                              GrMipmapped mipmapped,
                               const BackendTextureData* data) {
     int numMipLevels = 1;
-    if (mipMapped == GrMipMapped::kYes) {
-        numMipLevels = SkMipMap::ComputeLevelCount(dimensions.width(), dimensions.height()) + 1;
+    if (mipmapped == GrMipmapped::kYes) {
+        numMipLevels = SkMipmap::ComputeLevelCount(dimensions.width(), dimensions.height()) + 1;
     }
 
     if (!data || data->type() == BackendTextureData::Type::kColor) {
@@ -809,7 +813,7 @@ bool GrGpu::MipMapsAreCorrect(SkISize dimensions,
         return false;
     }
 
-    SkColorType colorType = data->pixmap(0).colorType();
+    GrColorType colorType = data->pixmap(0).colorType();
     for (int i = 1; i < numMipLevels; ++i) {
         dimensions = {std::max(1, dimensions.width()/2), std::max(1, dimensions.height()/2)};
         if (dimensions != data->pixmap(i).dimensions()) {
@@ -823,7 +827,7 @@ bool GrGpu::MipMapsAreCorrect(SkISize dimensions,
 }
 
 bool GrGpu::CompressedDataIsCorrect(SkISize dimensions, SkImage::CompressionType compressionType,
-                                    GrMipMapped mipMapped, const BackendTextureData* data) {
+                                    GrMipmapped mipMapped, const BackendTextureData* data) {
 
     if (!data || data->type() == BackendTextureData::Type::kColor) {
         return true;
@@ -836,7 +840,7 @@ bool GrGpu::CompressedDataIsCorrect(SkISize dimensions, SkImage::CompressionType
     SkASSERT(data->type() == BackendTextureData::Type::kCompressed);
 
     size_t computedSize = SkCompressedDataSize(compressionType, dimensions,
-                                               nullptr, mipMapped == GrMipMapped::kYes);
+                                               nullptr, mipMapped == GrMipmapped::kYes);
 
     return computedSize == data->compressedSize();
 }
@@ -844,7 +848,7 @@ bool GrGpu::CompressedDataIsCorrect(SkISize dimensions, SkImage::CompressionType
 GrBackendTexture GrGpu::createBackendTexture(SkISize dimensions,
                                              const GrBackendFormat& format,
                                              GrRenderable renderable,
-                                             GrMipMapped mipMapped,
+                                             GrMipmapped mipMapped,
                                              GrProtected isProtected) {
     const GrCaps* caps = this->caps();
 
@@ -862,7 +866,7 @@ GrBackendTexture GrGpu::createBackendTexture(SkISize dimensions,
         return {};
     }
 
-    if (mipMapped == GrMipMapped::kYes && !this->caps()->mipMapSupport()) {
+    if (mipMapped == GrMipmapped::kYes && !this->caps()->mipmapSupport()) {
         return {};
     }
 
@@ -870,59 +874,45 @@ GrBackendTexture GrGpu::createBackendTexture(SkISize dimensions,
 }
 
 bool GrGpu::updateBackendTexture(const GrBackendTexture& backendTexture,
-                                 GrGpuFinishedProc finishedProc,
-                                 GrGpuFinishedContext finishedContext,
+                                 sk_sp<GrRefCntedCallback> finishedCallback,
                                  const BackendTextureData* data) {
     SkASSERT(data);
     const GrCaps* caps = this->caps();
 
-    sk_sp<GrRefCntedCallback> callback;
-    if (finishedProc) {
-        callback.reset(new GrRefCntedCallback(finishedProc, finishedContext));
-    }
-
     if (!backendTexture.isValid()) {
         return false;
     }
 
     if (data->type() == BackendTextureData::Type::kPixmaps) {
-        auto ct = SkColorTypeToGrColorType(data->pixmap(0).colorType());
+        auto ct = data->pixmap(0).colorType();
         if (!caps->areColorTypeAndFormatCompatible(ct, backendTexture.getBackendFormat())) {
             return false;
         }
     }
 
-    if (backendTexture.hasMipMaps() && !this->caps()->mipMapSupport()) {
+    if (backendTexture.hasMipmaps() && !this->caps()->mipmapSupport()) {
         return false;
     }
 
-    GrMipMapped mipMapped = backendTexture.hasMipMaps() ? GrMipMapped::kYes : GrMipMapped::kNo;
-    if (!MipMapsAreCorrect(backendTexture.dimensions(), mipMapped, data)) {
+    GrMipmapped mipmapped = backendTexture.hasMipmaps() ? GrMipmapped::kYes : GrMipmapped::kNo;
+    if (!MipMapsAreCorrect(backendTexture.dimensions(), mipmapped, data)) {
         return false;
     }
 
-    return this->onUpdateBackendTexture(backendTexture, std::move(callback), data);
+    return this->onUpdateBackendTexture(backendTexture, std::move(finishedCallback), data);
 }
 
 GrBackendTexture GrGpu::createCompressedBackendTexture(SkISize dimensions,
                                                        const GrBackendFormat& format,
-                                                       GrMipMapped mipMapped,
-                                                       GrProtected isProtected,
-                                                       GrGpuFinishedProc finishedProc,
-                                                       GrGpuFinishedContext finishedContext,
-                                                       const BackendTextureData* data) {
-    sk_sp<GrRefCntedCallback> callback;
-    if (finishedProc) {
-        callback.reset(new GrRefCntedCallback(finishedProc, finishedContext));
-    }
-
+                                                       GrMipmapped mipMapped,
+                                                       GrProtected isProtected) {
     const GrCaps* caps = this->caps();
 
     if (!format.isValid()) {
         return {};
     }
 
-    SkImage::CompressionType compressionType = caps->compressionType(format);
+    SkImage::CompressionType compressionType = GrBackendFormatToCompressionType(format);
     if (compressionType == SkImage::CompressionType::kNone) {
         // Uncompressed formats must go through the createBackendTexture API
         return {};
@@ -934,73 +924,40 @@ GrBackendTexture GrGpu::createCompressedBackendTexture(SkISize dimensions,
         return {};
     }
 
-    if (mipMapped == GrMipMapped::kYes && !this->caps()->mipMapSupport()) {
+    if (mipMapped == GrMipmapped::kYes && !this->caps()->mipmapSupport()) {
         return {};
     }
 
-    if (!CompressedDataIsCorrect(dimensions, compressionType, mipMapped, data)) {
-        return {};
+    return this->onCreateCompressedBackendTexture(dimensions, format, mipMapped, isProtected);
+}
+
+bool GrGpu::updateCompressedBackendTexture(const GrBackendTexture& backendTexture,
+                                           sk_sp<GrRefCntedCallback> finishedCallback,
+                                           const BackendTextureData* data) {
+    SkASSERT(data);
+
+    if (!backendTexture.isValid()) {
+        return false;
     }
 
-    return this->onCreateCompressedBackendTexture(dimensions, format, mipMapped,
-                                                  isProtected, std::move(callback), data);
-}
+    GrBackendFormat format = backendTexture.getBackendFormat();
 
-GrStagingBuffer* GrGpu::findStagingBuffer(size_t size) {
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    for (auto b : fActiveStagingBuffers) {
-        if (b->remaining() >= size) {
-            return b;
-        }
+    SkImage::CompressionType compressionType = GrBackendFormatToCompressionType(format);
+    if (compressionType == SkImage::CompressionType::kNone) {
+        // Uncompressed formats must go through the createBackendTexture API
+        return false;
     }
-    for (auto b : fAvailableStagingBuffers) {
-        if (b->remaining() >= size) {
-            fAvailableStagingBuffers.remove(b);
-            fActiveStagingBuffers.addToTail(b);
-            return b;
-        }
+
+    if (backendTexture.hasMipmaps() && !this->caps()->mipmapSupport()) {
+        return false;
     }
-    size = SkNextPow2(size);
-    size = std::max(size, kMinStagingBufferSize);
-    std::unique_ptr<GrStagingBuffer> b = this->createStagingBuffer(size);
-    GrStagingBuffer* stagingBuffer = b.get();
-    fStagingBuffers.push_back(std::move(b));
-    fActiveStagingBuffers.addToTail(stagingBuffer);
-    return stagingBuffer;
-}
 
-GrStagingBuffer::Slice GrGpu::allocateStagingBufferSlice(size_t size) {
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    GrStagingBuffer* stagingBuffer = this->findStagingBuffer(size);
-    return stagingBuffer->allocate(size);
-}
+    GrMipmapped mipMapped = backendTexture.hasMipmaps() ? GrMipmapped::kYes : GrMipmapped::kNo;
 
-void GrGpu::unmapStagingBuffers() {
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    // Unmap all active buffers.
-    for (auto buffer : fActiveStagingBuffers) {
-        buffer->unmap();
+    if (!CompressedDataIsCorrect(backendTexture.dimensions(), compressionType, mipMapped, data)) {
+        return false;
     }
-}
 
-void GrGpu::moveStagingBufferFromBusyToAvailable(GrStagingBuffer* buffer) {
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    fBusyStagingBuffers.remove(buffer);
-    fAvailableStagingBuffers.addToTail(buffer);
-}
-
-void GrGpu::moveStagingBufferFromActiveToBusy(GrStagingBuffer* buffer) {
-#ifdef SK_DEBUG
-    this->validateStagingBuffers();
-#endif
-    fActiveStagingBuffers.remove(buffer);
-    fBusyStagingBuffers.addToTail(buffer);
+    return this->onUpdateCompressedBackendTexture(backendTexture, std::move(finishedCallback),
+                                                  data);
 }
