@@ -10,14 +10,15 @@
 #include "include/core/SkRect.h"
 #include "include/gpu/GrBackendSurface.h"
 #include "src/core/SkCompressedDataUtils.h"
+#include "src/gpu/GrBackendUtils.h"
 #include "src/gpu/GrProcessor.h"
 #include "src/gpu/GrProgramDesc.h"
 #include "src/gpu/GrProgramInfo.h"
 #include "src/gpu/GrRenderTarget.h"
-#include "src/gpu/GrRenderTargetPriv.h"
 #include "src/gpu/GrRenderTargetProxy.h"
 #include "src/gpu/GrShaderCaps.h"
 #include "src/gpu/GrSurfaceProxy.h"
+#include "src/gpu/mtl/GrMtlRenderTarget.h"
 #include "src/gpu/mtl/GrMtlUtil.h"
 
 #if !__has_feature(objc_arc)
@@ -128,10 +129,29 @@ void GrMtlCaps::initFeatureSet(MTLFeatureSet featureSet) {
     SK_ABORT("Requested an unsupported feature set");
 }
 
-bool GrMtlCaps::canCopyAsBlit(GrSurface* dst, int dstSampleCount,
-                              GrSurface* src, int srcSampleCount,
-                              const SkIRect& srcRect, const SkIPoint& dstPoint,
-                              bool areDstSrcSameObj) const {
+static int get_surface_sample_cnt(GrSurface* surf) {
+    if (const GrRenderTarget* rt = surf->asRenderTarget()) {
+        return rt->numSamples();
+    }
+    return 1;
+}
+
+static bool is_resolving_msaa(GrSurface* surf) {
+    auto rt = static_cast<GrMtlRenderTarget*>(surf->asRenderTarget());
+    if (rt && rt->mtlResolveTexture()) {
+        SkASSERT(rt->numSamples() > 1);
+        return true;
+    }
+    return false;
+}
+
+bool GrMtlCaps::canCopyAsBlit(GrSurface* dst,
+                              GrSurface* src,
+                              const SkIRect& srcRect,
+                              const SkIPoint& dstPoint) const {
+    if (is_resolving_msaa(src) || is_resolving_msaa(dst)) {
+        return false;
+    }
     id<MTLTexture> dstTex = GrGetMTLTextureFromSurface(dst);
     id<MTLTexture> srcTex = GrGetMTLTextureFromSurface(src);
     if (srcTex.framebufferOnly || dstTex.framebufferOnly) {
@@ -140,9 +160,11 @@ bool GrMtlCaps::canCopyAsBlit(GrSurface* dst, int dstSampleCount,
 
     MTLPixelFormat dstFormat = dstTex.pixelFormat;
     MTLPixelFormat srcFormat = srcTex.pixelFormat;
+    int srcSampleCount = get_surface_sample_cnt(src);
+    int dstSampleCount = get_surface_sample_cnt(dst);
 
-    return this->canCopyAsBlit(dstFormat, dstSampleCount, srcFormat, srcSampleCount,
-                               srcRect, dstPoint, areDstSrcSameObj);
+    return this->canCopyAsBlit(dstFormat, dstSampleCount, srcFormat, srcSampleCount, srcRect,
+                               dstPoint, src == dst);
 }
 
 bool GrMtlCaps::canCopyAsBlit(MTLPixelFormat dstFormat, int dstSampleCount,
@@ -165,16 +187,35 @@ bool GrMtlCaps::canCopyAsBlit(MTLPixelFormat dstFormat, int dstSampleCount,
     return true;
 }
 
-bool GrMtlCaps::canCopyAsResolve(GrSurface* dst, int dstSampleCount,
-                                 GrSurface* src, int srcSampleCount,
-                                 const SkIRect& srcRect, const SkIPoint& dstPoint) const {
-    if (dst == src) {
+bool GrMtlCaps::canCopyAsResolve(GrSurface* dst,
+                                 GrSurface* src,
+                                 const SkIRect& srcRect,
+                                 const SkIPoint& dstPoint) const {
+    MTLPixelFormat dstFormat = GrBackendFormatAsMTLPixelFormat(dst->backendFormat());
+    MTLPixelFormat srcFormat = GrBackendFormatAsMTLPixelFormat(src->backendFormat());
+
+    int srcSampleCount = get_surface_sample_cnt(src);
+    int dstSampleCount = get_surface_sample_cnt(dst);
+
+    bool srcIsRenderTarget = src->asRenderTarget();
+    SkISize srcSize = src->dimensions();
+    return this->canCopyAsResolve(dstFormat, dstSampleCount, srcFormat, srcSampleCount,
+                                  srcIsRenderTarget, srcSize, srcRect, dstPoint, src == dst);
+}
+
+bool GrMtlCaps::canCopyAsResolve(MTLPixelFormat dstFormat, int dstSampleCount,
+                                 MTLPixelFormat srcFormat, int srcSampleCount,
+                                 bool srcIsRenderTarget, const SkISize srcDimensions,
+                                 const SkIRect& srcRect,
+                                 const SkIPoint& dstPoint,
+                                 bool areDstSrcSameObj) const {
+    if (areDstSrcSameObj) {
         return false;
     }
-    if (dst->backendFormat() != src->backendFormat()) {
+    if (dstFormat != srcFormat) {
         return false;
     }
-    if (dstSampleCount > 1 || srcSampleCount == 1 || !src->asRenderTarget()) {
+    if (dstSampleCount > 1 || srcSampleCount == 1 || !srcIsRenderTarget) {
         return false;
     }
 
@@ -182,7 +223,7 @@ bool GrMtlCaps::canCopyAsResolve(GrSurface* dst, int dstSampleCount,
     if (dstPoint != SkIPoint::Make(0, 0)) {
         return false;
     }
-    if (srcRect != SkIRect::MakeXYWH(0, 0, src->width(), src->height())) {
+    if (srcRect != SkIRect::MakeSize(srcDimensions)) {
         return false;
     }
 
@@ -191,22 +232,30 @@ bool GrMtlCaps::canCopyAsResolve(GrSurface* dst, int dstSampleCount,
 
 bool GrMtlCaps::onCanCopySurface(const GrSurfaceProxy* dst, const GrSurfaceProxy* src,
                                  const SkIRect& srcRect, const SkIPoint& dstPoint) const {
-    int dstSampleCnt = 0;
-    int srcSampleCnt = 0;
+    int dstSampleCnt = 1;
+    int srcSampleCnt = 1;
     if (const GrRenderTargetProxy* rtProxy = dst->asRenderTargetProxy()) {
         dstSampleCnt = rtProxy->numSamples();
     }
     if (const GrRenderTargetProxy* rtProxy = src->asRenderTargetProxy()) {
         srcSampleCnt = rtProxy->numSamples();
     }
-    SkASSERT((dstSampleCnt > 0) == SkToBool(dst->asRenderTargetProxy()));
-    SkASSERT((srcSampleCnt > 0) == SkToBool(src->asRenderTargetProxy()));
 
     // TODO: need some way to detect whether the proxy is framebufferOnly
 
-    return this->canCopyAsBlit(GrBackendFormatAsMTLPixelFormat(dst->backendFormat()), dstSampleCnt,
-                               GrBackendFormatAsMTLPixelFormat(src->backendFormat()), srcSampleCnt,
-                               srcRect, dstPoint, dst == src);
+    if (this->canCopyAsBlit(GrBackendFormatAsMTLPixelFormat(dst->backendFormat()), dstSampleCnt,
+                            GrBackendFormatAsMTLPixelFormat(src->backendFormat()), srcSampleCnt,
+                            srcRect, dstPoint, dst == src)) {
+        return true;
+    }
+    bool srcIsRenderTarget = src->asRenderTargetProxy();
+    MTLPixelFormat dstFormat = GrBackendFormatAsMTLPixelFormat(dst->backendFormat());
+    MTLPixelFormat srcFormat = GrBackendFormatAsMTLPixelFormat(src->backendFormat());
+    return this->canCopyAsResolve(dstFormat, dstSampleCnt,
+                                  srcFormat, srcSampleCnt,
+                                  srcIsRenderTarget, src->backingStoreDimensions(), srcRect,
+                                  dstPoint,
+                                  dst == src);
 }
 
 void GrMtlCaps::initGrCaps(const id<MTLDevice> device) {
@@ -238,6 +287,8 @@ void GrMtlCaps::initGrCaps(const id<MTLDevice> device) {
     fMaxPreferredRenderTargetSize = fMaxRenderTargetSize;
     fMaxTextureSize = fMaxRenderTargetSize;
 
+    fMaxPushConstantsSize = 4*1024;
+
     // Init sample counts. All devices support 1 (i.e. 0 in skia).
     fSampleCounts.push_back(1);
     if (@available(iOS 9.0, *)) {
@@ -264,7 +315,7 @@ void GrMtlCaps::initGrCaps(const id<MTLDevice> device) {
 
     fOversizedStencilSupport = true;
 
-    fMipMapSupport = true;   // always available in Metal
+    fMipmapSupport = true;   // always available in Metal
     fNPOTTextureTileSupport = true;  // always available in Metal
 
     fReuseScratchTextures = true; // Assuming this okay
@@ -280,12 +331,8 @@ void GrMtlCaps::initGrCaps(const id<MTLDevice> device) {
     if (@available(macOS 10.11, iOS 9.0, *)) {
         if (this->isMac() || 3 == fFamilyGroup) {
             fDrawInstancedSupport = true;
+            fNativeDrawIndirectSupport = true;
         }
-        // https://developer.apple.com/documentation/metal/mtlrendercommandencoder/specifying_drawing_and_dispatch_arguments_indirectly
-        // Metal does not appear to have a call that issues multiple indirect draws. Furthermore,
-        // the indirect draw structs are ordered differently than GL and Vulkan. For now we leave it
-        // unsupported and rely on polyfills in GrOpsRenderPass.
-        SkASSERT(!fNativeDrawIndirectSupport);
     }
 
     fMixedSamplesSupport = false;
@@ -318,24 +365,6 @@ bool GrMtlCaps::isFormatSRGB(const GrBackendFormat& format) const {
     return format_is_srgb(GrBackendFormatAsMTLPixelFormat(format));
 }
 
-SkImage::CompressionType GrMtlCaps::compressionType(const GrBackendFormat& format) const {
-
-    switch (GrBackendFormatAsMTLPixelFormat(format)) {
-#ifdef SK_BUILD_FOR_IOS
-        case MTLPixelFormatETC2_RGB8:
-            // ETC2 uses the same compression layout as ETC1
-            return SkImage::CompressionType::kETC2_RGB8_UNORM;
-#else
-        case MTLPixelFormatBC1_RGBA:
-            return SkImage::CompressionType::kBC1_RGBA8_UNORM;
-#endif
-        default:
-            return SkImage::CompressionType::kNone;
-    }
-
-    SkUNREACHABLE;
-}
-
 bool GrMtlCaps::isFormatTexturable(const GrBackendFormat& format) const {
     MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(format);
     return this->isFormatTexturable(mtlFormat);
@@ -406,15 +435,6 @@ int GrMtlCaps::getRenderTargetSampleCount(int requestedCount, MTLPixelFormat for
     return 1 == requestedCount ? 1 : 0;
 }
 
-size_t GrMtlCaps::bytesPerPixel(const GrBackendFormat& format) const {
-    MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(format);
-    return this->bytesPerPixel(mtlFormat);
-}
-
-size_t GrMtlCaps::bytesPerPixel(MTLPixelFormat format) const {
-    return this->getFormatInfo(format).fBytesPerPixel;
-}
-
 void GrMtlCaps::initShaderCaps() {
     GrShaderCaps* shaderCaps = fShaderCaps.get();
 
@@ -455,6 +475,10 @@ void GrMtlCaps::initShaderCaps() {
     shaderCaps->fMaxFragmentSamplers = 16;
 }
 
+// Define this so we can use it to initialize arrays and work around
+// the fact that MTLPixelFormatBGR10A2Unorm is not always available.
+#define kMTLPixelFormatBGR10A2Unorm MTLPixelFormat(94)
+
 // These are all the valid MTLPixelFormats that we support in Skia.  They are roughly ordered from
 // most frequently used to least to improve look up times in arrays.
 static constexpr MTLPixelFormat kMtlFormats[] = {
@@ -470,8 +494,7 @@ static constexpr MTLPixelFormat kMtlFormats[] = {
     MTLPixelFormatRG8Unorm,
     MTLPixelFormatRGB10A2Unorm,
 #ifdef SK_BUILD_FOR_MAC
-    // BGR10_A2 wasn't added until iOS 11
-    MTLPixelFormatBGR10A2Unorm,
+    kMTLPixelFormatBGR10A2Unorm,
 #endif
 #ifdef SK_BUILD_FOR_IOS
     MTLPixelFormatABGR4Unorm,
@@ -534,11 +557,14 @@ size_t GrMtlCaps::GetFormatIndex(MTLPixelFormat pixelFormat) {
 void GrMtlCaps::initFormatTable() {
     FormatInfo* info;
 
+    if (@available(macos 10.13, ios 11.0, *)) {
+        SkASSERT(kMTLPixelFormatBGR10A2Unorm == MTLPixelFormatBGR10A2Unorm);
+    }
+
     // Format: R8Unorm
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatR8Unorm)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 1;
         info->fColorTypeInfoCount = 2;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -547,8 +573,8 @@ void GrMtlCaps::initFormatTable() {
             auto& ctInfo = info->fColorTypeInfos[ctIdx++];
             ctInfo.fColorType = GrColorType::kAlpha_8;
             ctInfo.fFlags = ColorTypeInfo::kUploadData_Flag | ColorTypeInfo::kRenderable_Flag;
-            ctInfo.fReadSwizzle = GrSwizzle::RRRR();
-            ctInfo.fWriteSwizzle = GrSwizzle::AAAA();
+            ctInfo.fReadSwizzle = GrSwizzle("000r");
+            ctInfo.fWriteSwizzle = GrSwizzle("a000");
         }
         // Format: R8Unorm, Surface: kGray_8
         {
@@ -563,7 +589,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatA8Unorm)];
         info->fFlags = FormatInfo::kTexturable_Flag;
-        info->fBytesPerPixel = 1;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -572,7 +597,6 @@ void GrMtlCaps::initFormatTable() {
             auto& ctInfo = info->fColorTypeInfos[ctIdx++];
             ctInfo.fColorType = GrColorType::kAlpha_8;
             ctInfo.fFlags = ColorTypeInfo::kUploadData_Flag | ColorTypeInfo::kRenderable_Flag;
-            ctInfo.fReadSwizzle = GrSwizzle::AAAA();
         }
     }
 
@@ -581,7 +605,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatB5G6R5Unorm)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 2;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -597,7 +620,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatABGR4Unorm)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 2;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -614,7 +636,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRGBA8Unorm)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 2;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -637,7 +658,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRG8Unorm)];
         info->fFlags = FormatInfo::kTexturable_Flag;
-        info->fBytesPerPixel = 2;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -653,7 +673,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatBGRA8Unorm)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -669,7 +688,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRGBA8Unorm_sRGB)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -689,7 +707,6 @@ void GrMtlCaps::initFormatTable() {
         } else {
             info->fFlags = FormatInfo::kTexturable_Flag;
         }
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -703,14 +720,13 @@ void GrMtlCaps::initFormatTable() {
 
 #ifdef SK_BUILD_FOR_MAC
     // Format: BGR10A2Unorm
-    {
+    if (@available(macos 10.13, ios 11.0, *)) {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatBGR10A2Unorm)];
         if (this->isMac() && fFamilyGroup == 1) {
             info->fFlags = FormatInfo::kTexturable_Flag;
         } else {
             info->fFlags = FormatInfo::kAllFlags;
         }
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -727,7 +743,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatR16Float)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 2;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -736,8 +751,8 @@ void GrMtlCaps::initFormatTable() {
             auto& ctInfo = info->fColorTypeInfos[ctIdx++];
             ctInfo.fColorType = GrColorType::kAlpha_F16;
             ctInfo.fFlags = ColorTypeInfo::kUploadData_Flag | ColorTypeInfo::kRenderable_Flag;
-            ctInfo.fReadSwizzle = GrSwizzle::RRRR();
-            ctInfo.fWriteSwizzle = GrSwizzle::AAAA();
+            ctInfo.fReadSwizzle = GrSwizzle("000r");
+            ctInfo.fWriteSwizzle = GrSwizzle("a000");
         }
     }
 
@@ -745,7 +760,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRGBA16Float)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 8;
         info->fColorTypeInfoCount = 2;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -771,7 +785,6 @@ void GrMtlCaps::initFormatTable() {
         } else {
             info->fFlags = FormatInfo::kTexturable_Flag | FormatInfo::kRenderable_Flag;
         }
-        info->fBytesPerPixel = 2;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -780,8 +793,8 @@ void GrMtlCaps::initFormatTable() {
             auto& ctInfo = info->fColorTypeInfos[ctIdx++];
             ctInfo.fColorType = GrColorType::kAlpha_16;
             ctInfo.fFlags = ColorTypeInfo::kUploadData_Flag | ColorTypeInfo::kRenderable_Flag;
-            ctInfo.fReadSwizzle = GrSwizzle::RRRR();
-            ctInfo.fWriteSwizzle = GrSwizzle::AAAA();
+            ctInfo.fReadSwizzle = GrSwizzle("000r");
+            ctInfo.fWriteSwizzle = GrSwizzle("a000");
         }
     }
 
@@ -793,7 +806,6 @@ void GrMtlCaps::initFormatTable() {
         } else {
             info->fFlags = FormatInfo::kTexturable_Flag | FormatInfo::kRenderable_Flag;
         }
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -825,7 +837,6 @@ void GrMtlCaps::initFormatTable() {
         } else {
             info->fFlags = FormatInfo::kTexturable_Flag | FormatInfo::kRenderable_Flag;
         }
-        info->fBytesPerPixel = 8;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -841,7 +852,6 @@ void GrMtlCaps::initFormatTable() {
     {
         info = &fFormatTable[GetFormatIndex(MTLPixelFormatRG16Float)];
         info->fFlags = FormatInfo::kAllFlags;
-        info->fBytesPerPixel = 4;
         info->fColorTypeInfoCount = 1;
         info->fColorTypeInfos.reset(new ColorTypeInfo[info->fColorTypeInfoCount]());
         int ctIdx = 0;
@@ -873,7 +883,9 @@ void GrMtlCaps::initFormatTable() {
     this->setColorType(GrColorType::kBGRA_8888,        { MTLPixelFormatBGRA8Unorm });
     this->setColorType(GrColorType::kRGBA_1010102,     { MTLPixelFormatRGB10A2Unorm });
 #ifdef SK_BUILD_FOR_MAC
-    this->setColorType(GrColorType::kBGRA_1010102,     { MTLPixelFormatBGR10A2Unorm });
+    if (@available(macos 10.13, ios 11.0, *)) {
+        this->setColorType(GrColorType::kBGRA_1010102, { MTLPixelFormatBGR10A2Unorm });
+    }
 #endif
     this->setColorType(GrColorType::kGray_8,           { MTLPixelFormatR8Unorm });
     this->setColorType(GrColorType::kAlpha_F16,        { MTLPixelFormatR16Float });
@@ -886,7 +898,7 @@ void GrMtlCaps::initFormatTable() {
 }
 
 void GrMtlCaps::initStencilFormat(id<MTLDevice> physDev) {
-    fPreferredStencilFormat = StencilFormat{ MTLPixelFormatStencil8, 8, 8, true };
+    fPreferredStencilFormat = MTLPixelFormatStencil8;
 }
 
 bool GrMtlCaps::onSurfaceSupportsWritePixels(const GrSurface* surface) const {
@@ -896,16 +908,35 @@ bool GrMtlCaps::onSurfaceSupportsWritePixels(const GrSurface* surface) const {
     return true;
 }
 
+GrCaps::SurfaceReadPixelsSupport GrMtlCaps::surfaceSupportsReadPixels(
+        const GrSurface* surface) const {
+    if (auto mtlRT = static_cast<const GrMtlRenderTarget*>(surface->asRenderTarget())) {
+        if (mtlRT->numSamples() > 1 && !mtlRT->mtlResolveTexture()) {
+            return SurfaceReadPixelsSupport::kCopyToTexture2D;
+        }
+    }
+    return SurfaceReadPixelsSupport::kSupported;
+}
+
+GrCaps::DstCopyRestrictions GrMtlCaps::getDstCopyRestrictions(const GrRenderTargetProxy* src,
+                                                              GrColorType ct) const {
+    // If the src is a MSAA RT then the only supported copy action (not considering falling back
+    // to a draw) is to resolve from the MSAA src to the non-MSAA dst. Currently we only support
+    // resolving the entire texture to a resolve buffer of the same size.
+    DstCopyRestrictions restrictions = {};
+    if (auto rtProxy = src->asRenderTargetProxy()) {
+        if (rtProxy->numSamples() > 1) {
+            restrictions.fMustCopyWholeSrc = true;
+            restrictions.fRectsMustMatch = GrSurfaceProxy::RectsMustMatch::kYes;
+        }
+    }
+    return restrictions;
+}
+
 bool GrMtlCaps::onAreColorTypeAndFormatCompatible(GrColorType ct,
                                                   const GrBackendFormat& format) const {
     MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(format);
 
-    SkImage::CompressionType compression = GrMtlFormatToCompressionType(mtlFormat);
-    if (compression != SkImage::CompressionType::kNone) {
-        return ct == (SkCompressionTypeIsOpaque(compression) ? GrColorType::kRGB_888x
-                                                             : GrColorType::kRGBA_8888);
-    }
-
     const auto& info = this->getFormatInfo(mtlFormat);
     for (int i = 0; i < info.fColorTypeInfoCount; ++i) {
         if (info.fColorTypeInfos[i].fColorType == ct) {
@@ -948,7 +979,7 @@ GrBackendFormat GrMtlCaps::getBackendFormatFromCompressionType(
     SK_ABORT("Invalid compression type");
 }
 
-GrSwizzle GrMtlCaps::getReadSwizzle(const GrBackendFormat& format, GrColorType colorType) const {
+GrSwizzle GrMtlCaps::onGetReadSwizzle(const GrBackendFormat& format, GrColorType colorType) const {
     MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(format);
     SkASSERT(mtlFormat != MTLPixelFormatInvalid);
     const auto& info = this->getFormatInfo(mtlFormat);
@@ -1008,9 +1039,7 @@ GrCaps::SupportedWrite GrMtlCaps::supportedWritePixelsColorType(
 GrCaps::SupportedRead GrMtlCaps::onSupportedReadPixelsColorType(
         GrColorType srcColorType, const GrBackendFormat& srcBackendFormat,
         GrColorType dstColorType) const {
-    MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(srcBackendFormat);
-
-    SkImage::CompressionType compression = GrMtlFormatToCompressionType(mtlFormat);
+    SkImage::CompressionType compression = GrBackendFormatToCompressionType(srcBackendFormat);
     if (compression != SkImage::CompressionType::kNone) {
 #ifdef SK_BUILD_FOR_IOS
         // Reading back to kRGB_888x doesn't work on Metal/iOS (skbug.com/9839)
@@ -1024,6 +1053,7 @@ GrCaps::SupportedRead GrMtlCaps::onSupportedReadPixelsColorType(
     // Metal requires the destination offset for copyFromTexture to be a multiple of the textures
     // pixels size.
     size_t offsetAlignment = GrColorTypeBytesPerPixel(srcColorType);
+    MTLPixelFormat mtlFormat = GrBackendFormatAsMTLPixelFormat(srcBackendFormat);
 
     const auto& info = this->getFormatInfo(mtlFormat);
     for (int i = 0; i < info.fColorTypeInfoCount; ++i) {
@@ -1046,9 +1076,10 @@ GrCaps::SupportedRead GrMtlCaps::onSupportedReadPixelsColorType(
  * pipeline. This includes blending information and primitive type. The pipeline is immutable
  * so any remaining dynamic state is set via the MtlRenderCmdEncoder.
  */
-GrProgramDesc GrMtlCaps::makeDesc(const GrRenderTarget* rt,
-                                  const GrProgramInfo& programInfo) const {
-
+GrProgramDesc GrMtlCaps::makeDesc(GrRenderTarget* rt,
+                                  const GrProgramInfo& programInfo,
+                                  ProgramDescOverrideFlags overrideFlags) const {
+    SkASSERT(overrideFlags == ProgramDescOverrideFlags::kNone);
     GrProgramDesc desc;
     if (!GrProgramDesc::Build(&desc, rt, programInfo, *this)) {
         SkASSERT(!desc.isValid());
@@ -1062,15 +1093,14 @@ GrProgramDesc GrMtlCaps::makeDesc(const GrRenderTarget* rt,
     b.add32(programInfo.numRasterSamples());
 
 #ifdef SK_DEBUG
-    if (rt && programInfo.pipeline().isStencilEnabled()) {
-        SkASSERT(rt->renderTargetPriv().getStencilAttachment());
+    if (rt && programInfo.isStencilEnabled()) {
+        SkASSERT(rt->getStencilAttachment());
     }
 #endif
 
-    b.add32(rt && rt->renderTargetPriv().getStencilAttachment()
-                                 ? this->preferredStencilFormat().fInternalFormat
-                                 : MTLPixelFormatInvalid);
-    b.add32((uint32_t)programInfo.pipeline().isStencilEnabled());
+    b.add32(rt && rt->getStencilAttachment() ? this->preferredStencilFormat()
+                                             : MTLPixelFormatInvalid);
+    b.add32((uint32_t)programInfo.isStencilEnabled());
     // Stencil samples don't seem to be tracked in the MTLRenderPipeline
 
     programInfo.pipeline().genKey(&b, *this);
@@ -1080,7 +1110,6 @@ GrProgramDesc GrMtlCaps::makeDesc(const GrRenderTarget* rt,
     return desc;
 }
 
-
 #if GR_TEST_UTILS
 std::vector<GrCaps::TestFormatColorTypeCombination> GrMtlCaps::getTestingCombinations() const {
     std::vector<GrCaps::TestFormatColorTypeCombination> combos = {
@@ -1102,7 +1131,7 @@ std::vector<GrCaps::TestFormatColorTypeCombination> GrMtlCaps::getTestingCombina
         { GrColorType::kBGRA_8888,        GrBackendFormat::MakeMtl(MTLPixelFormatBGRA8Unorm)      },
         { GrColorType::kRGBA_1010102,     GrBackendFormat::MakeMtl(MTLPixelFormatRGB10A2Unorm)    },
 #ifdef SK_BUILD_FOR_MAC
-        { GrColorType::kBGRA_1010102,     GrBackendFormat::MakeMtl(MTLPixelFormatBGR10A2Unorm)    },
+        { GrColorType::kBGRA_1010102,     GrBackendFormat::MakeMtl(kMTLPixelFormatBGR10A2Unorm)   },
 #endif
         { GrColorType::kGray_8,           GrBackendFormat::MakeMtl(MTLPixelFormatR8Unorm)         },
         { GrColorType::kAlpha_F16,        GrBackendFormat::MakeMtl(MTLPixelFormatR16Float)        },
@@ -1127,8 +1156,8 @@ void GrMtlCaps::onDumpJSON(SkJSONWriter* writer) const {
     writer->beginObject("Metal caps");
 
     writer->beginObject("Preferred Stencil Format");
-    writer->appendS32("stencil bits", fPreferredStencilFormat.fStencilBits);
-    writer->appendS32("total bits", fPreferredStencilFormat.fTotalBits);
+    writer->appendS32("stencil bits", GrMtlFormatStencilBits(fPreferredStencilFormat));
+    writer->appendS32("total bytes", GrMtlFormatBytesPerBlock(fPreferredStencilFormat));
     writer->endObject();
 
     switch (fPlatform) {
