@@ -317,10 +317,11 @@ private:
 
     void onCreateProgramInfo(const GrCaps* caps,
                              SkArenaAlloc* arena,
-                             const GrSurfaceProxyView* writeView,
+                             const GrSurfaceProxyView& writeView,
                              GrAppliedClip&& appliedClip,
                              const GrXferProcessor::DstProxyView& dstProxyView,
-                             GrXferBarrierFlags renderPassXferBarriers) override {
+                             GrXferBarrierFlags renderPassXferBarriers,
+                             GrLoadOp colorLoadOp) override {
 
         DashCap capType = (this->cap() == SkPaint::kRound_Cap) ? kRound_DashCap : kNonRound_DashCap;
 
@@ -360,6 +361,7 @@ private:
                                                                    std::move(fProcessorSet),
                                                                    GrPrimitiveType::kTriangles,
                                                                    renderPassXferBarriers,
+                                                                   colorLoadOp,
                                                                    pipelineFlags,
                                                                    fStencilSettings);
     }
@@ -835,7 +837,6 @@ public:
 
 private:
     friend class GLDashingCircleEffect;
-    friend class ::SkArenaAlloc; // for access to ctor
 
     DashingCircleEffect(const SkPMColor4f&, AAMode aaMode, const SkMatrix& localMatrix,
                         bool usesLocalCoords);
@@ -967,7 +968,9 @@ GrGeometryProcessor* DashingCircleEffect::Make(SkArenaAlloc* arena,
                                                AAMode aaMode,
                                                const SkMatrix& localMatrix,
                                                bool usesLocalCoords) {
-    return arena->make<DashingCircleEffect>(color, aaMode, localMatrix, usesLocalCoords);
+    return arena->make([&](void* ptr) {
+        return new (ptr) DashingCircleEffect(color, aaMode, localMatrix, usesLocalCoords);
+    });
 }
 
 void DashingCircleEffect::getGLSLProcessorKey(const GrShaderCaps& caps,
@@ -1045,7 +1048,6 @@ public:
 
 private:
     friend class GLDashingLineEffect;
-    friend class ::SkArenaAlloc; // for access to ctor
 
     DashingLineEffect(const SkPMColor4f&, AAMode aaMode, const SkMatrix& localMatrix,
                       bool usesLocalCoords);
@@ -1193,7 +1195,9 @@ GrGeometryProcessor* DashingLineEffect::Make(SkArenaAlloc* arena,
                                              AAMode aaMode,
                                              const SkMatrix& localMatrix,
                                              bool usesLocalCoords) {
-    return arena->make<DashingLineEffect>(color, aaMode, localMatrix, usesLocalCoords);
+    return arena->make([&](void* ptr) {
+        return new (ptr) DashingLineEffect(color, aaMode, localMatrix, usesLocalCoords);
+    });
 }
 
 void DashingLineEffect::getGLSLProcessorKey(const GrShaderCaps& caps,
