@@ -11,15 +11,16 @@
 
 #if SK_SUPPORT_GPU && defined(SK_BUILD_FOR_ANDROID) && __ANDROID_API__ >= 26 && defined(SK_VULKAN)
 
+#include "include/core/SkBitmap.h"
 #include "include/core/SkCanvas.h"
 #include "include/core/SkImage.h"
 #include "include/core/SkSurface.h"
 #include "include/gpu/GrBackendSemaphore.h"
-#include "include/gpu/GrContext.h"
+#include "include/gpu/GrDirectContext.h"
 #include "include/gpu/vk/GrVkBackendContext.h"
 #include "include/gpu/vk/GrVkExtensions.h"
 #include "src/core/SkAutoMalloc.h"
-#include "src/gpu/GrContextPriv.h"
+#include "src/gpu/GrDirectContextPriv.h"
 #include "src/gpu/GrGpu.h"
 #include "src/gpu/GrProxyProvider.h"
 #include "src/gpu/SkGr.h"
@@ -60,7 +61,7 @@ public:
 
     virtual void makeCurrent() = 0;
 
-    virtual GrContext* grContext() = 0;
+    virtual GrDirectContext* directContext() = 0;
 
     int getFdHandle() { return fFdHandle; }
 
@@ -110,7 +111,7 @@ public:
 
     void makeCurrent() override { fGLCtx->makeCurrent(); }
 
-    GrContext* grContext() override { return fGrContext; }
+    GrDirectContext* directContext() override { return fDirectContext; }
 
 private:
     bool importHardwareBuffer(skiatest::Reporter* reporter, AHardwareBuffer* buffer);
@@ -136,14 +137,14 @@ private:
     sk_gpu_test::ContextInfo fGLESContextInfo;
 
     sk_gpu_test::GLTestContext* fGLCtx = nullptr;
-    GrContext* fGrContext = nullptr;
+    GrDirectContext* fDirectContext = nullptr;
 };
 
 bool EGLTestHelper::init(skiatest::Reporter* reporter) {
     fGLESContextInfo = fFactory.getContextInfo(sk_gpu_test::GrContextFactory::kGLES_ContextType);
-    fGrContext = fGLESContextInfo.grContext();
+    fDirectContext = fGLESContextInfo.directContext();
     fGLCtx = fGLESContextInfo.glContext();
-    if (!fGrContext || !fGLCtx) {
+    if (!fDirectContext || !fGLCtx) {
         return false;
     }
 
@@ -217,7 +218,7 @@ bool EGLTestHelper::init(skiatest::Reporter* reporter) {
 }
 
 bool EGLTestHelper::importHardwareBuffer(skiatest::Reporter* reporter, AHardwareBuffer* buffer) {
-    GrGLClearErr(fGLCtx->gl());
+    while (fGLCtx->gl()->fFunctions.fGetError() != GR_GL_NO_ERROR) {}
 
     EGLClientBuffer eglClientBuffer = fEGLGetNativeClientBufferANDROID(buffer);
     EGLint eglAttribs[] = { EGL_IMAGE_PRESERVED_KHR, EGL_TRUE,
@@ -237,19 +238,18 @@ bool EGLTestHelper::importHardwareBuffer(skiatest::Reporter* reporter, AHardware
         return false;
     }
     GR_GL_CALL_NOERRCHECK(fGLCtx->gl(), BindTexture(GR_GL_TEXTURE_2D, fTexID));
-    if (GR_GL_GET_ERROR(fGLCtx->gl()) != GR_GL_NO_ERROR) {
+    if (fGLCtx->gl()->fFunctions.fGetError() != GR_GL_NO_ERROR) {
         ERRORF(reporter, "Failed to bind GL Texture");
         return false;
     }
 
     fEGLImageTargetTexture2DOES(GL_TEXTURE_2D, fImage);
-    GLenum status = GL_NO_ERROR;
-    if ((status = glGetError()) != GL_NO_ERROR) {
-        ERRORF(reporter, "EGLImageTargetTexture2DOES failed (%#x)", (int) status);
+    if (GrGLenum error = fGLCtx->gl()->fFunctions.fGetError(); error != GR_GL_NO_ERROR) {
+        ERRORF(reporter, "EGLImageTargetTexture2DOES failed (%#x)", (int) error);
         return false;
     }
 
-    fGrContext->resetContext(kTextureBinding_GrGLBackendState);
+    fDirectContext->resetContext(kTextureBinding_GrGLBackendState);
     return true;
 }
 
@@ -263,15 +263,15 @@ sk_sp<SkImage> EGLTestHelper::importHardwareBufferForRead(skiatest::Reporter* re
     textureInfo.fID = fTexID;
     textureInfo.fFormat = GR_GL_RGBA8;
 
-    GrBackendTexture backendTex(DEV_W, DEV_H, GrMipMapped::kNo, textureInfo);
+    GrBackendTexture backendTex(DEV_W, DEV_H, GrMipmapped::kNo, textureInfo);
     REPORTER_ASSERT(reporter, backendTex.isValid());
 
-    sk_sp<SkImage> image = SkImage::MakeFromTexture(fGrContext,
-                                                               backendTex,
-                                                               kTopLeft_GrSurfaceOrigin,
-                                                               kRGBA_8888_SkColorType,
-                                                               kPremul_SkAlphaType,
-                                                               nullptr);
+    sk_sp<SkImage> image = SkImage::MakeFromTexture(fDirectContext,
+                                                    backendTex,
+                                                    kTopLeft_GrSurfaceOrigin,
+                                                    kRGBA_8888_SkColorType,
+                                                    kPremul_SkAlphaType,
+                                                    nullptr);
 
     if (!image) {
         ERRORF(reporter, "Failed to make wrapped GL SkImage");
@@ -291,10 +291,10 @@ sk_sp<SkSurface> EGLTestHelper::importHardwareBufferForWrite(skiatest::Reporter*
     textureInfo.fID = fTexID;
     textureInfo.fFormat = GR_GL_RGBA8;
 
-    GrBackendTexture backendTex(DEV_W, DEV_H, GrMipMapped::kNo, textureInfo);
+    GrBackendTexture backendTex(DEV_W, DEV_H, GrMipmapped::kNo, textureInfo);
     REPORTER_ASSERT(reporter, backendTex.isValid());
 
-    sk_sp<SkSurface> surface = SkSurface::MakeFromBackendTexture(fGrContext,
+    sk_sp<SkSurface> surface = SkSurface::MakeFromBackendTexture(fDirectContext,
                                                                  backendTex,
                                                                  kTopLeft_GrSurfaceOrigin,
                                                                  0,
@@ -318,7 +318,7 @@ bool EGLTestHelper::flushSurfaceAndSignalSemaphore(skiatest::Reporter* reporter,
         return false;
     }
 
-    surface->flush();
+    surface->flushAndSubmit();
     GR_GL_CALL(fGLCtx->gl(), Flush());
     fFdHandle = fEGLDupNativeFenceFDANDROID(eglDisplay, eglsync);
 
@@ -358,9 +358,8 @@ bool EGLTestHelper::importAndWaitOnSemaphore(skiatest::Reporter* reporter, int f
 }
 
 void EGLTestHelper::doClientSync() {
-    GrFlushInfo flushInfo;
-    flushInfo.fFlags = kSyncCpu_GrFlushFlag;
-    this->grContext()->flush(flushInfo);
+    this->directContext()->flush();
+    this->directContext()->submit(true);
 }
 #endif  // SK_GL
 
@@ -406,7 +405,7 @@ public:
         }
     }
     void cleanup() override {
-        fGrContext.reset();
+        fDirectContext.reset();
         this->releaseImage();
         if (fSignalSemaphore != VK_NULL_HANDLE) {
             fVkDestroySemaphore(fDevice, fSignalSemaphore, nullptr);
@@ -437,11 +436,11 @@ public:
     bool init(skiatest::Reporter* reporter) override;
 
     void doClientSync() override {
-        if (!fGrContext) {
+        if (!fDirectContext) {
             return;
         }
 
-        fGrContext->priv().getGpu()->testingOnly_flushGpuAndSync();
+        fDirectContext->submit(true);
     }
 
     bool flushSurfaceAndSignalSemaphore(skiatest::Reporter* reporter, sk_sp<SkSurface>) override;
@@ -456,7 +455,7 @@ public:
 
     void makeCurrent() override {}
 
-    GrContext* grContext() override { return fGrContext.get(); }
+    GrDirectContext* directContext() override { return fDirectContext.get(); }
 
 private:
     bool checkOptimalHardwareBuffer(skiatest::Reporter* reporter);
@@ -504,7 +503,7 @@ private:
     VkDevice fDevice = VK_NULL_HANDLE;
 
     GrVkBackendContext fBackendContext;
-    sk_sp<GrContext> fGrContext;
+    sk_sp<GrDirectContext> fDirectContext;
 };
 
 bool VulkanTestHelper::init(skiatest::Reporter* reporter) {
@@ -578,9 +577,9 @@ bool VulkanTestHelper::init(skiatest::Reporter* reporter) {
     ACQUIRE_DEVICE_VK_PROC(ImportSemaphoreFdKHR);
     ACQUIRE_DEVICE_VK_PROC(DestroySemaphore);
 
-    fGrContext = GrContext::MakeVulkan(fBackendContext);
-    REPORTER_ASSERT(reporter, fGrContext.get());
-    if (!fGrContext) {
+    fDirectContext = GrDirectContext::MakeVulkan(fBackendContext);
+    REPORTER_ASSERT(reporter, fDirectContext.get());
+    if (!fDirectContext) {
         return false;
     }
 
@@ -785,11 +784,18 @@ bool VulkanTestHelper::importHardwareBuffer(skiatest::Reporter* reporter,
         return false;
     }
 
+    GrVkAlloc alloc;
+    alloc.fMemory = fMemory;
+    alloc.fOffset = 0;
+    alloc.fSize = hwbProps.allocationSize;
+    alloc.fFlags = 0;
+
     outImageInfo->fImage = fImage;
-    outImageInfo->fAlloc = GrVkAlloc(fMemory, 0, hwbProps.allocationSize, 0);
+    outImageInfo->fAlloc = alloc;
     outImageInfo->fImageTiling = VK_IMAGE_TILING_OPTIMAL;
     outImageInfo->fImageLayout = VK_IMAGE_LAYOUT_UNDEFINED;
     outImageInfo->fFormat = VK_FORMAT_R8G8B8A8_UNORM;
+    outImageInfo->fImageUsageFlags = usageFlags;
     outImageInfo->fLevelCount = 1;
     outImageInfo->fCurrentQueueFamily = VK_QUEUE_FAMILY_EXTERNAL;
     return true;
@@ -804,7 +810,7 @@ sk_sp<SkImage> VulkanTestHelper::importHardwareBufferForRead(skiatest::Reporter*
 
     GrBackendTexture backendTex(DEV_W, DEV_H, imageInfo);
 
-    sk_sp<SkImage> wrappedImage = SkImage::MakeFromTexture(fGrContext.get(),
+    sk_sp<SkImage> wrappedImage = SkImage::MakeFromTexture(fDirectContext.get(),
                                                            backendTex,
                                                            kTopLeft_GrSurfaceOrigin,
                                                            kRGBA_8888_SkColorType,
@@ -821,7 +827,7 @@ sk_sp<SkImage> VulkanTestHelper::importHardwareBufferForRead(skiatest::Reporter*
 
 bool VulkanTestHelper::flushSurfaceAndSignalSemaphore(skiatest::Reporter* reporter,
                                                       sk_sp<SkSurface> surface) {
-    surface->flush();
+    surface->flushAndSubmit();
     surface.reset();
     GrBackendSemaphore semaphore;
     if (!this->setupSemaphoreForSignaling(reporter, &semaphore)) {
@@ -830,9 +836,10 @@ bool VulkanTestHelper::flushSurfaceAndSignalSemaphore(skiatest::Reporter* report
     GrFlushInfo info;
     info.fNumSemaphores = 1;
     info.fSignalSemaphores = &semaphore;
-    GrSemaphoresSubmitted submitted = fGrContext->flush(info);
+    GrSemaphoresSubmitted submitted = fDirectContext->flush(info);
+    fDirectContext->submit();
     if (GrSemaphoresSubmitted::kNo == submitted) {
-        ERRORF(reporter, "Failing call to flush on GrContext");
+        ERRORF(reporter, "Failing call to flush on GrDirectContext");
         return false;
     }
     SkASSERT(semaphore.isInitialized());
@@ -965,7 +972,7 @@ sk_sp<SkSurface> VulkanTestHelper::importHardwareBufferForWrite(skiatest::Report
 
     GrBackendTexture backendTex(DEV_W, DEV_H, imageInfo);
 
-    sk_sp<SkSurface> surface = SkSurface::MakeFromBackendTexture(fGrContext.get(),
+    sk_sp<SkSurface> surface = SkSurface::MakeFromBackendTexture(fDirectContext.get(),
                                                                  backendTex,
                                                                  kTopLeft_GrSurfaceOrigin,
                                                                  0,
@@ -1240,13 +1247,13 @@ void run_test(skiatest::Reporter* reporter, const GrContextOptions& options,
         return;
     }
 
-    GrContext* grContext = dstHelper->grContext();
+    auto direct = dstHelper->directContext();
 
     // Make SkSurface to render wrapped HWB into.
     SkImageInfo imageInfo = SkImageInfo::Make(DEV_W, DEV_H, kRGBA_8888_SkColorType,
                                               kPremul_SkAlphaType, nullptr);
 
-    sk_sp<SkSurface> dstSurf = SkSurface::MakeRenderTarget(grContext,
+    sk_sp<SkSurface> dstSurf = SkSurface::MakeRenderTarget(direct,
                                                            SkBudgeted::kNo, imageInfo, 0,
                                                            kTopLeft_GrSurfaceOrigin,
                                                            nullptr, false);
