@@ -36,13 +36,13 @@ public:
         kClipRRect_OpType,
         kClipShader_OpType,
         kConcat_OpType,
+        kConcat44_OpType,
         kDrawAnnotation_OpType,
         kDrawBitmap_OpType,
         kDrawBitmapRect_OpType,
         kDrawDRRect_OpType,
         kDrawImage_OpType,
         kDrawImageLattice_OpType,
-        kDrawImageNine_OpType,
         kDrawImageRect_OpType,
         kDrawImageRectLayer_OpType, // unique to DebugCanvas
         kDrawOval_OpType,
@@ -66,8 +66,9 @@ public:
         kSave_OpType,
         kSaveLayer_OpType,
         kSetMatrix_OpType,
+        kSetM44_OpType,
 
-        kLast_OpType = kSetMatrix_OpType
+        kLast_OpType = kSetM44_OpType
     };
 
     static const int kOpTypeCount = kLast_OpType + 1;
@@ -99,6 +100,7 @@ public:
     static void MakeJsonRect(SkJSONWriter&, const SkRect& rect);
     static void MakeJsonIRect(SkJSONWriter&, const SkIRect&);
     static void MakeJsonMatrix(SkJSONWriter&, const SkMatrix&);
+    static void MakeJsonMatrix44(SkJSONWriter&, const SkM44&);
     static void MakeJsonPath(SkJSONWriter&, const SkPath& path);
     static void MakeJsonRegion(SkJSONWriter&, const SkRegion& region);
     static void MakeJsonPaint(SkJSONWriter&, const SkPaint& paint, UrlDataManager& urlDataManager);
@@ -111,6 +113,7 @@ public:
     static bool flatten(const SkBitmap& bitmap,
                         SkJSONWriter&   writer,
                         UrlDataManager& urlDataManager);
+    OpType getOpType() const { return fOpType; }
 
 private:
     OpType fOpType;
@@ -123,7 +126,7 @@ public:
     void execute(SkCanvas* canvas) const override;
 
 private:
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClearCommand : public DrawCommand {
@@ -135,7 +138,7 @@ public:
 private:
     SkColor fColor;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClipPathCommand : public DrawCommand {
@@ -150,7 +153,7 @@ private:
     SkClipOp fOp;
     bool     fDoAA;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClipRegionCommand : public DrawCommand {
@@ -163,7 +166,7 @@ private:
     SkRegion fRegion;
     SkClipOp fOp;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClipRectCommand : public DrawCommand {
@@ -177,7 +180,7 @@ private:
     SkClipOp fOp;
     bool     fDoAA;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClipRRectCommand : public DrawCommand {
@@ -192,7 +195,7 @@ private:
     SkClipOp fOp;
     bool     fDoAA;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ClipShaderCommand : public DrawCommand {
@@ -206,7 +209,7 @@ private:
     sk_sp<SkShader> fShader;
     SkClipOp fOp;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class ConcatCommand : public DrawCommand {
@@ -218,7 +221,19 @@ public:
 private:
     SkMatrix fMatrix;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
+};
+
+class Concat44Command : public DrawCommand {
+public:
+    Concat44Command(const SkM44& matrix);
+    void execute(SkCanvas* canvas) const override;
+    void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
+
+private:
+    SkM44 fMatrix;
+
+    using INHERITED = DrawCommand;
 };
 
 class DrawAnnotationCommand : public DrawCommand {
@@ -232,23 +247,26 @@ private:
     SkString      fKey;
     sk_sp<SkData> fValue;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawImageCommand : public DrawCommand {
 public:
-    DrawImageCommand(const SkImage* image, SkScalar left, SkScalar top, const SkPaint* paint);
+    DrawImageCommand(const SkImage* image, SkScalar left, SkScalar top,
+                     const SkSamplingOptions&, const SkPaint* paint);
     void execute(SkCanvas* canvas) const override;
     bool render(SkCanvas* canvas) const override;
     void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
+    uint64_t imageId(UrlDataManager& udb) const;
 
 private:
     sk_sp<const SkImage> fImage;
     SkScalar             fLeft;
     SkScalar             fTop;
+    SkSamplingOptions    fSampling;
     SkTLazy<SkPaint>     fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawImageLatticeCommand : public DrawCommand {
@@ -256,58 +274,45 @@ public:
     DrawImageLatticeCommand(const SkImage*           image,
                             const SkCanvas::Lattice& lattice,
                             const SkRect&            dst,
+                            SkFilterMode,
                             const SkPaint*           paint);
     void execute(SkCanvas* canvas) const override;
     bool render(SkCanvas* canvas) const override;
     void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
+    uint64_t imageId(UrlDataManager& udb) const;
 
 private:
     sk_sp<const SkImage> fImage;
     SkCanvas::Lattice    fLattice;
     SkRect               fDst;
+    SkFilterMode         fFilter;
     SkTLazy<SkPaint>     fPaint;
 
-    typedef DrawCommand INHERITED;
-};
-
-class DrawImageNineCommand : public DrawCommand {
-public:
-    DrawImageNineCommand(const SkImage* image,
-                         const SkIRect& center,
-                         const SkRect&  dst,
-                         const SkPaint* paint);
-    void execute(SkCanvas* canvas) const override;
-    bool render(SkCanvas* canvas) const override;
-    void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
-
-private:
-    sk_sp<const SkImage> fImage;
-    SkIRect              fCenter;
-    SkRect               fDst;
-    SkTLazy<SkPaint>     fPaint;
-
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawImageRectCommand : public DrawCommand {
 public:
     DrawImageRectCommand(const SkImage*              image,
-                         const SkRect*               src,
+                         const SkRect&               src,
                          const SkRect&               dst,
+                         const SkSamplingOptions&    sampling,
                          const SkPaint*              paint,
                          SkCanvas::SrcRectConstraint constraint);
     void execute(SkCanvas* canvas) const override;
     bool render(SkCanvas* canvas) const override;
     void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
+    uint64_t imageId(UrlDataManager& udm) const;
 
 private:
     sk_sp<const SkImage>        fImage;
-    SkTLazy<SkRect>             fSrc;
+    SkRect                      fSrc;
     SkRect                      fDst;
+    SkSamplingOptions           fSampling;
     SkTLazy<SkPaint>            fPaint;
     SkCanvas::SrcRectConstraint fConstraint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 // Command for resolving the deferred SkImage representing an android layer
@@ -318,8 +323,9 @@ public:
     DrawImageRectLayerCommand(DebugLayerManager*          layerManager,
                               const int                   nodeId,
                               const int                   frame,
-                              const SkRect*               src,
+                              const SkRect&               src,
                               const SkRect&               dst,
+                              const SkSamplingOptions&    sampling,
                               const SkPaint*              paint,
                               SkCanvas::SrcRectConstraint constraint);
     void execute(SkCanvas* canvas) const override;
@@ -330,12 +336,13 @@ private:
     DebugLayerManager*          fLayerManager;
     int                         fNodeId;
     int                         fFrame;
-    SkTLazy<SkRect>             fSrc;
+    SkRect                      fSrc;
     SkRect                      fDst;
+    SkSamplingOptions           fSampling;
     SkTLazy<SkPaint>            fPaint;
     SkCanvas::SrcRectConstraint fConstraint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawOvalCommand : public DrawCommand {
@@ -349,7 +356,7 @@ private:
     SkRect  fOval;
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawArcCommand : public DrawCommand {
@@ -370,7 +377,7 @@ private:
     bool     fUseCenter;
     SkPaint  fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawPaintCommand : public DrawCommand {
@@ -383,7 +390,7 @@ public:
 private:
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawBehindCommand : public DrawCommand {
@@ -396,7 +403,7 @@ public:
 private:
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawPathCommand : public DrawCommand {
@@ -410,7 +417,7 @@ private:
     SkPath  fPath;
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class BeginDrawPictureCommand : public DrawCommand {
@@ -425,7 +432,7 @@ private:
     SkTLazy<SkMatrix>      fMatrix;
     SkTLazy<SkPaint>       fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class EndDrawPictureCommand : public DrawCommand {
@@ -437,7 +444,7 @@ public:
 private:
     bool fRestore;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawPointsCommand : public DrawCommand {
@@ -455,7 +462,7 @@ private:
     SkTDArray<SkPoint>  fPts;
     SkPaint             fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawRegionCommand : public DrawCommand {
@@ -469,7 +476,7 @@ private:
     SkRegion fRegion;
     SkPaint  fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawTextBlobCommand : public DrawCommand {
@@ -486,7 +493,7 @@ private:
     SkScalar          fYPos;
     SkPaint           fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawPatchCommand : public DrawCommand {
@@ -508,7 +515,7 @@ private:
     SkBlendMode fBlendMode;
     SkPaint     fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawRectCommand : public DrawCommand {
@@ -521,7 +528,7 @@ private:
     SkRect  fRect;
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawRRectCommand : public DrawCommand {
@@ -535,7 +542,7 @@ private:
     SkRRect fRRect;
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawDRRectCommand : public DrawCommand {
@@ -550,7 +557,7 @@ private:
     SkRRect fInner;
     SkPaint fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawVerticesCommand : public DrawCommand {
@@ -564,7 +571,7 @@ private:
     SkBlendMode       fBlendMode;
     SkPaint           fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawAtlasCommand : public DrawCommand {
@@ -575,6 +582,7 @@ public:
                      const SkColor[],
                      int,
                      SkBlendMode,
+                     const SkSamplingOptions&,
                      const SkRect*,
                      const SkPaint*);
 
@@ -586,10 +594,11 @@ private:
     SkTDArray<SkRect>    fTex;
     SkTDArray<SkColor>   fColors;
     SkBlendMode          fBlendMode;
+    SkSamplingOptions    fSampling;
     SkTLazy<SkRect>      fCull;
     SkTLazy<SkPaint>     fPaint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class SaveCommand : public DrawCommand {
@@ -598,7 +607,7 @@ public:
     void execute(SkCanvas* canvas) const override;
 
 private:
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class SaveLayerCommand : public DrawCommand {
@@ -613,7 +622,7 @@ private:
     sk_sp<const SkImageFilter> fBackdrop;
     uint32_t                   fSaveLayerFlags;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class SetMatrixCommand : public DrawCommand {
@@ -625,7 +634,19 @@ public:
 private:
     SkMatrix fMatrix;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
+};
+
+class SetM44Command : public DrawCommand {
+public:
+    SetM44Command(const SkM44& matrix);
+    void execute(SkCanvas* canvas) const override;
+    void toJSON(SkJSONWriter& writer, UrlDataManager& urlDataManager) const override;
+
+private:
+    SkM44 fMatrix;
+
+    using INHERITED = DrawCommand;
 };
 
 class DrawShadowCommand : public DrawCommand {
@@ -639,7 +660,7 @@ private:
     SkPath          fPath;
     SkDrawShadowRec fShadowRec;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawDrawableCommand : public DrawCommand {
@@ -651,7 +672,7 @@ private:
     sk_sp<SkDrawable> fDrawable;
     SkTLazy<SkMatrix> fMatrix;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawEdgeAAQuadCommand : public DrawCommand {
@@ -671,7 +692,7 @@ private:
     SkColor4f             fColor;
     SkBlendMode           fMode;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 
 class DrawEdgeAAImageSetCommand : public DrawCommand {
@@ -680,6 +701,7 @@ public:
                               int count,
                               const SkPoint[],
                               const SkMatrix[],
+                              const SkSamplingOptions&,
                               const SkPaint*,
                               SkCanvas::SrcRectConstraint);
     void execute(SkCanvas* canvas) const override;
@@ -689,9 +711,10 @@ private:
     int                                   fCount;
     SkAutoTArray<SkPoint>                 fDstClips;
     SkAutoTArray<SkMatrix>                fPreViewMatrices;
+    SkSamplingOptions                     fSampling;
     SkTLazy<SkPaint>                      fPaint;
     SkCanvas::SrcRectConstraint           fConstraint;
 
-    typedef DrawCommand INHERITED;
+    using INHERITED = DrawCommand;
 };
 #endif
