@@ -7,11 +7,13 @@
 #ifndef XFA_FXFA_CXFA_FFPAGEVIEW_H_
 #define XFA_FXFA_CXFA_FFPAGEVIEW_H_
 
-#include <memory>
 #include <vector>
 
-#include "core/fxcrt/observed_ptr.h"
-#include "core/fxcrt/retain_ptr.h"
+#include "fxjs/gc/heap.h"
+#include "v8/include/cppgc/garbage-collected.h"
+#include "v8/include/cppgc/member.h"
+#include "v8/include/cppgc/visitor.h"
+#include "xfa/fxfa/fxfa.h"
 #include "xfa/fxfa/layout/cxfa_contentlayoutitem.h"
 #include "xfa/fxfa/layout/cxfa_traversestrategy_layoutitem.h"
 #include "xfa/fxfa/layout/cxfa_viewlayoutitem.h"
@@ -19,36 +21,43 @@
 class CXFA_FFWidget;
 class CXFA_FFDocView;
 
-class CXFA_FFPageView : public Observable {
+class CXFA_FFPageView final : public cppgc::GarbageCollected<CXFA_FFPageView> {
  public:
-  CXFA_FFPageView(CXFA_FFDocView* pDocView, CXFA_Node* pPageArea);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_FFPageView();
 
-  CXFA_ViewLayoutItem* GetLayoutItem() const { return m_pLayoutItem.Get(); }
+  void Trace(cppgc::Visitor* visitor) const;
+
+  CXFA_ViewLayoutItem* GetLayoutItem() const { return m_pLayoutItem; }
   void SetLayoutItem(CXFA_ViewLayoutItem* pItem) { m_pLayoutItem = pItem; }
 
   CXFA_FFDocView* GetDocView() const;
   CFX_RectF GetPageViewRect() const;
   CFX_Matrix GetDisplayMatrix(const FX_RECT& rtDisp, int32_t iRotate) const;
 
-  // These always return a non-null iterator.
-  std::unique_ptr<IXFA_WidgetIterator> CreateFormWidgetIterator(
-      uint32_t dwWidgetFilter);
-  std::unique_ptr<IXFA_WidgetIterator> CreateTraverseWidgetIterator(
+  // These always return a non-null iterator from the gc heap.
+  IXFA_WidgetIterator* CreateGCedFormWidgetIterator(uint32_t dwWidgetFilter);
+  IXFA_WidgetIterator* CreateGCedTraverseWidgetIterator(
       uint32_t dwWidgetFilter);
 
  private:
-  UnownedPtr<CXFA_Node> const m_pPageArea;
-  UnownedPtr<CXFA_FFDocView> const m_pDocView;
-  UnownedPtr<CXFA_ViewLayoutItem> m_pLayoutItem;
+  CXFA_FFPageView(CXFA_FFDocView* pDocView, CXFA_Node* pPageArea);
+
+  cppgc::Member<CXFA_Node> const m_pPageArea;
+  cppgc::Member<CXFA_FFDocView> const m_pDocView;
+  cppgc::Member<CXFA_ViewLayoutItem> m_pLayoutItem;
 };
 
-class CXFA_FFPageWidgetIterator final : public IXFA_WidgetIterator {
+class CXFA_FFPageWidgetIterator final
+    : public cppgc::GarbageCollected<CXFA_FFPageWidgetIterator>,
+      public IXFA_WidgetIterator {
  public:
-  CXFA_FFPageWidgetIterator(CXFA_FFPageView* pPageView, uint32_t dwFilter);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_FFPageWidgetIterator() override;
 
-  void Reset() override;
+  void Trace(cppgc::Visitor* visitor) const {}
+
+  // IXFA_WidgetIterator:
   CXFA_FFWidget* MoveToFirst() override;
   CXFA_FFWidget* MoveToLast() override;
   CXFA_FFWidget* MoveToNext() override;
@@ -57,37 +66,23 @@ class CXFA_FFPageWidgetIterator final : public IXFA_WidgetIterator {
   bool SetCurrentWidget(CXFA_FFWidget* hWidget) override;
 
  private:
-  CXFA_FFWidget* GetWidget(CXFA_LayoutItem* pLayoutItem);
+  CXFA_FFPageWidgetIterator(CXFA_FFPageView* pPageView, uint32_t dwFilter);
 
-  CXFA_LayoutItemIterator m_sIterator;  // Must outlive |m_pPageView|.
-  UnownedPtr<CXFA_FFPageView> const m_pPageView;
-  UnownedPtr<CXFA_FFWidget> m_hCurWidget;
+  CXFA_LayoutItemIterator m_sIterator;
   const uint32_t m_dwFilter;
   const bool m_bIgnoreRelevant;
 };
 
-class CXFA_TabParam {
+class CXFA_FFTabOrderPageWidgetIterator final
+    : public cppgc::GarbageCollected<CXFA_FFTabOrderPageWidgetIterator>,
+      public IXFA_WidgetIterator {
  public:
-  explicit CXFA_TabParam(CXFA_FFWidget* pWidget);
-  ~CXFA_TabParam();
-
-  void AppendTabParam(CXFA_TabParam* pParam);
-  void ClearChildren();
-  CXFA_FFWidget* GetWidget() const { return m_pWidget.Get(); }
-  const std::vector<CXFA_FFWidget*>& GetChildren() const { return m_Children; }
-
- private:
-  UnownedPtr<CXFA_FFWidget> const m_pWidget;
-  std::vector<CXFA_FFWidget*> m_Children;
-};
-
-class CXFA_FFTabOrderPageWidgetIterator final : public IXFA_WidgetIterator {
- public:
-  CXFA_FFTabOrderPageWidgetIterator(CXFA_FFPageView* pPageView,
-                                    uint32_t dwFilter);
+  CONSTRUCT_VIA_MAKE_GARBAGE_COLLECTED;
   ~CXFA_FFTabOrderPageWidgetIterator() override;
 
-  void Reset() override;
+  void Trace(cppgc::Visitor* visitor) const;
+
+  // IXFA_WidgetIterator:
   CXFA_FFWidget* MoveToFirst() override;
   CXFA_FFWidget* MoveToLast() override;
   CXFA_FFWidget* MoveToNext() override;
@@ -96,21 +91,17 @@ class CXFA_FFTabOrderPageWidgetIterator final : public IXFA_WidgetIterator {
   bool SetCurrentWidget(CXFA_FFWidget* hWidget) override;
 
  private:
+  CXFA_FFTabOrderPageWidgetIterator(CXFA_FFPageView* pPageView,
+                                    uint32_t dwFilter);
+
   CXFA_FFWidget* GetTraverseWidget(CXFA_FFWidget* pWidget);
   CXFA_FFWidget* FindWidgetByName(const WideString& wsWidgetName,
                                   CXFA_FFWidget* pRefWidget);
   void CreateTabOrderWidgetArray();
-  void CreateSpaceOrderWidgetArray(std::vector<CXFA_FFWidget*>* WidgetArray);
-  CXFA_FFWidget* GetWidget(CXFA_LayoutItem* pLayoutItem);
-  void OrderContainer(CXFA_LayoutItemIterator* sIterator,
-                      CXFA_LayoutItem* pViewItem,
-                      CXFA_TabParam* pContainer,
-                      bool* bCurrentItem,
-                      bool* bContentArea,
-                      bool bMasterPage);
+  std::vector<CXFA_ContentLayoutItem*> CreateSpaceOrderLayoutItems();
 
-  std::vector<RetainPtr<CXFA_ContentLayoutItem>> m_TabOrderWidgetArray;
-  UnownedPtr<CXFA_FFPageView> const m_pPageView;
+  cppgc::Member<CXFA_ViewLayoutItem> const m_pPageViewLayout;
+  std::vector<cppgc::Member<CXFA_ContentLayoutItem>> m_TabOrderWidgetArray;
   const uint32_t m_dwFilter;
   int32_t m_iCurWidget = -1;
   const bool m_bIgnoreRelevant;
